# -*- coding: utf-8 -*-


from django.db import models
from django.contrib.auth import get_user_model
from django.utils.translation import ugettext as _

from aparnik.contrib.basemodels.models import BaseModel
from aparnik.contrib.reviews.models import BaseReview, BaseReviewManager
from aparnik.contrib.filefields.models import FileField

User = get_user_model()


# QA MANAGER
class QAManager(BaseReviewManager):

    def get_queryset(self):
        return super(QAManager, self).get_queryset()
    
    def active(self, user=None):
        return super(QAManager, self).active(user)

    def get_this_user(self, user):
        return super(QAManager, self).get_this_user(user=user)

    def model_question_answer(self, model_obj, user_obj=None, with_children=False):
        return super(QAManager, self).model_base_reviews(model_obj, user_obj, with_children=with_children)

    def model_question_answer_count(self, model_obj, user_obj=None):
        return self.model_question_answer(model_obj, user_obj, with_children=True).count()


# QA MODEL
class QA(BaseReview):
    # ابجکتی که نظر براش ثبت شده به عنوان مثال کالای اول
    model_obj = models.ForeignKey(BaseModel, related_name='question_answers_set', on_delete=models.CASCADE, verbose_name=_('Model'))
    files = models.ManyToManyField(FileField, blank=True, verbose_name=_('Files'))

    objects = QAManager()

    class Meta:
        verbose_name = _('Question Answer')
        verbose_name_plural = _('Questions Answers')

    def __str__(self):
        return self.title



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-25 15:43


import aparnik.utils.fields
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('basemodels', '0001_initial'),
        ('sliders', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Product',
            fields=[
                ('basemodel_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='basemodels.BaseModel')),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
                ('price_fabric', aparnik.utils.fields.PriceField(decimal_places=0, max_digits=8, verbose_name='Price Fabric')),
                ('discount_percent', models.PositiveIntegerField(default=0, validators=[django.core.validators.MaxValueValidator(100), django.core.validators.MinValueValidator(0)], verbose_name='Discount Price')),
                ('slider_obj', models.ManyToManyField(related_name='products_sliders', to='sliders.Slider', verbose_name='Slider')),
            ],
            options={
                'verbose_name': 'Product',
                'verbose_name_plural': 'Products',
            },
            bases=('basemodels.basemodel',),
        ),
    ]



# -*- coding: utf-8 -*-


from django.db.models import Sum
from django.core.management import BaseCommand
from django.contrib.auth import get_user_model
from django.utils.timezone import now
from dateutil.relativedelta import relativedelta
from aparnik.packages.educations.courses.models import CourseFile, Course, CourseSummary, FileField
import datetime

User = get_user_model()


class Command(BaseCommand):
    # Show this when the user types help
    help = "calculate for index course"

    # A command must define handle()
    def handle(self, *args, **options):

        start_time = now()

        #
        reindex = False
        if Course.objects.update_needed().count() > 0:
            reindex = True

        if CourseFile.objects.update_needed().count() > 0:
            reindex = True

        if reindex:
            self.reindex_course()
            from aparnik.packages.educations.progresses.models import Progresses
            Progresses.objects.all().update(update_needed=True)

        # Course.objects.update()
        Course.objects.update_needed().update(update_needed=False)
        CourseFile.objects.update_needed().update(update_needed=False)
        finished_time = now()

        print(('reindex courses %s - time long: %ss.' % (now(), relativedelta(finished_time, start_time).seconds)))

    def reindex_course(self):

        for course in Course.objects.active():
            courses_id = Course.objects.children_deep(course).values_list('id', flat=True).distinct()

            # calculate type time
            for type_tuple in FileField.FILE_TYPE:
                type = type_tuple[0]
                files = CourseFile.objects.get_this_type(type=type).filter(course_obj_id__in=courses_id)
                time = files.aggregate(total_time_seconds=Sum('seconds'))['total_time_seconds'] or 0.0

                total_times = course.total_times.filter(type=type)
                total_time = None
                is_save = False
                if total_times.count() == 0:

                    total_time = CourseSummary.objects.create(total_time_seconds=time, file_count=files.count(),
                                                              file_count_preview=files.filter(is_free_field=True).count(),
                                                              type=type, course=course)
                    total_time.save()
                    is_save = True
                else:
                    total_time = total_times.first()
                    old_total_seconds = total_time.total_time_seconds
                    if old_total_seconds != time.__str__() or total_time.file_count != files.count() or total_time.file_count_preview != files.filter(
                            is_free_field=True).count():
                        total_time.total_time_seconds = time
                        total_time.file_count = files.count()
                        total_time.file_count_preview = files.filter(is_free_field=True).count()
                        total_time.save()
                        is_save = True

                if is_save:
                    course.save()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-28 21:16


from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('segments', '0002_auto_20181026_1745'),
        ('sliders', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='SliderSegment',
            fields=[
                ('basesegment_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='segments.BaseSegment')),
            ],
            options={
                'verbose_name': 'Slider Segment',
                'verbose_name_plural': 'Sliders Segments',
            },
            bases=('segments.basesegment',),
        ),
    ]



# -*- coding: utf-8 -*-


from rest_framework.generics import RetrieveAPIView, CreateAPIView, UpdateAPIView, ListAPIView
from rest_framework.permissions import (AllowAny, IsAuthenticated)

from ..models import BaseSegment
from .serializers import BaseSegmentListPolymorphicSerializer
from aparnik.contrib.basemodels.api.serializers import ModelListPolymorphicSerializer


class BaseSegmentListAPIView(ListAPIView):
    serializer_class = BaseSegmentListPolymorphicSerializer
    permission_classes = [AllowAny]
    # filter_backends = (filters.SearchFilter,)
    search_fields = ('id',)

    def get_queryset(self):
        queryset = BaseSegment.objects.all()
        if 'segment_id' in self.request.parser_context['kwargs']:
            queryset = BaseSegment.objects.filter(id=self.request.parser_context['kwargs']['segment_id'])
        return queryset


class BaseSegmentModelListAPIView(ListAPIView):
    serializer_class = ModelListPolymorphicSerializer
    permission_classes = [AllowAny]
    # filter_backends = (filters.SearchFilter,)
    search_fields = ('id',)

    def get_queryset(self):
        if 'segment_id' in self.request.parser_context['kwargs']:
            segment = BaseSegment.objects.filter(id=self.request.parser_context['kwargs']['segment_id']).first()
            queryset = segment.model_obj.all()
        return queryset



# -*- coding: utf-8 -*-


from rest_framework.generics import RetrieveAPIView, CreateAPIView, UpdateAPIView, ListAPIView
from rest_framework.permissions import (AllowAny, IsAuthenticated)

from aparnik.contrib.basemodels.api.serializers import ModelListPolymorphicSerializer
from ..models import Teacher
from .serializers import TeacherListPolymorphicSerializer


# Create your views here.
class TeacherListAPIView(ListAPIView):
    serializer_class = TeacherListPolymorphicSerializer
    queryset = Teacher.objects.active()
    permission_classes = [AllowAny]


class TeacherModelsListAPIView(ListAPIView):
    serializer_class = ModelListPolymorphicSerializer
    permission_classes = [AllowAny]

    def get_queryset(self):
        queryset = Teacher.objects.active()
        if 'teacher_id' in self.request.parser_context['kwargs']:
            try:
                teacher = Teacher.objects.get(id=self.request.parser_context['kwargs']['teacher_id'])
                return teacher.get_models()
            except Exception as e:
                pass
        return queryset



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-28 16:12


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pages', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='page',
            name='english_title',
            field=models.CharField(default='home', max_length=100, verbose_name='English Title'),
            preserve_default=False,
        ),
    ]



from django.conf.urls import url, include
from rest_framework_jwt.views import obtain_jwt_token
from rest_framework_jwt.views import verify_jwt_token

from .views import SupportListAPIView, SupportDetailAPIView

app_name = 'supports'

urlpatterns = [
    url(r'^$', SupportListAPIView.as_view(), name='list'),
    url(r'^(?P<id>\d+)/$', SupportDetailAPIView.as_view(), name='detail'),

]


# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-02-01 19:15


import aparnik.utils.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('orders', '0009_orderitem_description'),
    ]

    operations = [
        migrations.AddField(
            model_name='order',
            name='postal_cost_value',
            field=aparnik.utils.fields.PriceField(decimal_places=0, default=0, max_digits=20, verbose_name='Postal Cost'),
        ),
    ]



# -*- coding: utf-8 -*-


from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class CoSalesConfig(AppConfig):
    name = 'aparnik.packages.shops.cosales'
    verbose_name = _('Co Sales')



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-12-03 13:27


from django.db import migrations

def empty_table(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    SocialNetwork = apps.get_model('socials', 'SocialNetwork')
    SocialNetwork.objects.all().delete()


class Migration(migrations.Migration):

    dependencies = [
        ('socials', '0002_socialnetworksegment'),
    ]

    operations = [
        migrations.RunPython(empty_table, reverse_code=migrations.RunPython.noop),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-26 17:45


from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('files', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='file',
            options={'ordering': ['-created_at'], 'verbose_name': '\\u0641\\u0627\\u06cc\\u0644', 'verbose_name_plural': '\\u0641\\u0627\\u06cc\\u0644 \\u0647\\u0627'},
        ),
        migrations.AlterField(
            model_name='file',
            name='banner',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='shop_file_banner', to='filefields.FileField', verbose_name='\\u0628\\u0646\\u0631'),
        ),
        migrations.AlterField(
            model_name='file',
            name='description',
            field=models.TextField(blank=True, null=True, verbose_name='\\u062a\\u0648\\u0636\\u06cc\\u062d\\u0627\\u062a'),
        ),
        migrations.AlterField(
            model_name='file',
            name='file_obj',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='shop_file_obj', to='filefields.FileField', verbose_name='\\u0641\\u0627\\u06cc\\u0644'),
        ),
        migrations.AlterField(
            model_name='file',
            name='is_preview',
            field=models.BooleanField(default=False, verbose_name='\\u067e\\u06cc\\u0634 \\u0646\\u0645\\u0627\\u06cc\\u0634\\u061f'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-25 15:43


from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('basemodels', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Page',
            fields=[
                ('basemodel_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='basemodels.BaseModel')),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
            ],
            options={
                'verbose_name': 'Page',
                'verbose_name_plural': 'Pages',
            },
            bases=('basemodels.basemodel',),
        ),
    ]



from rest_framework import serializers

from aparnik.utils.utils import is_app_installed
from aparnik.contrib.basemodels.api.serializers import BaseModelListSerializer, BaseModelDetailSerializer, \
    ModelListPolymorphicSerializer
from aparnik.contrib.filefields.api.serializers import FileFieldListSerailizer
from ..models import Button


class ButtonListSerializer(BaseModelListSerializer):
    icon = serializers.SerializerMethodField()
    model = serializers.SerializerMethodField()

    class Meta:
        model = Button
        fields = BaseModelListSerializer.Meta.fields + [
            'title',
            'icon',
            'background_color',
            'icon_color',
            'title_color',
            'model',
        ]
        read_only_fields = BaseModelListSerializer.Meta.read_only_fields + ['title',
                                                                            'icon',
                                                                            'background_color',
                                                                            'icon_color',
                                                                            'title_color',
                                                                            'model',
                                                                            ]

    def get_icon(self, obj):
        return FileFieldListSerailizer(obj.icon, many=False, read_only=True, context=self.context).data if obj.icon \
            else None

    def get_model(self, obj):
        return ModelListPolymorphicSerializer(obj.model_obj.get_real_instance(), many=False, read_only=True, context=self.context).data


class ButtonDetailsSerializer(ButtonListSerializer, BaseModelDetailSerializer):
    class Meta:
        model = Button
        fields = ButtonListSerializer.Meta.fields + BaseModelDetailSerializer.Meta.fields + [

        ]


if is_app_installed('aparnik.contrib.segments'):
    from aparnik.contrib.segments.api.serializers import BaseSegmentListSerializer, BaseSegmentDetailSerializer
    from ..models import ButtonSegment

    # Base Segment ListSegmentReview
    class SegmentButtonListSerializer(BaseSegmentListSerializer):

        class Meta:
            model = ButtonSegment
            fields = BaseSegmentListSerializer.Meta.fields + [

            ]
            read_only_fields = BaseSegmentListSerializer.Meta.read_only_fields + []


    # Book Segment Details
    class SegmentButtonDetailSerializer(BaseSegmentDetailSerializer, SegmentButtonListSerializer):

        class Meta:
            model = ButtonSegment
            fields = SegmentButtonListSerializer.Meta.fields + BaseSegmentDetailSerializer.Meta.fields + [

            ]
            read_only_fields = BaseSegmentDetailSerializer.Meta.read_only_fields + []



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-12-10 14:32


from django.db import migrations


def command(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    Product = apps.get_model('products', 'Product')
    Setting = apps.get_model('settings', 'Setting')
    key = 'PRODUCT_WALLET_ID'
    value_id = Setting.objects.get(key=key).value
    wallet = Product.objects.get(id=value_id)
    wallet.is_show_only_for_super_user = True
    wallet.save()


class Migration(migrations.Migration):

    dependencies = [
        ('settings', '0005_auto_20181130_1938'),
    ]

    operations = [
        migrations.RunPython(command, reverse_code=migrations.RunPython.noop),
    ]



'''
Module: orderedmapping.py
Version: 1.0

Author:
   Taro Ogawa (tso@users.sourceforge.org)
   
Copyright:
    Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.

Licence:
    This module is distributed under the Lesser General Public Licence.
    http://www.opensource.org/licenses/lgpl-license.php
'''



class OrderedMapping(dict):
    def __init__(self, *pairs):
        self.order = []
        for key, val in pairs:
            self[key] = val

    def __setitem__(self, key, val):
        if key not in self:
            self.order.append(key)
        super(OrderedMapping, self).__setitem__(key, val)

    def __iter__(self):
        for item in self.order:
            yield item

    def __repr__(self):
        out = ["%s: %s" % (repr(item), repr(self[item])) for item in self]
        out = ", ".join(out)
        return "{%s}" % out



# -*- coding: utf-8 -*-


from django.contrib import admin

from aparnik.utils.utils import is_app_installed
from aparnik.contrib.users.admin import get_update_at, get_user_search_fields
from aparnik.contrib.basemodels.admin import BaseModelAdmin, BaseModelTabularInline
from .models import Notification, NotificationForSingleUser, User, Membership, NotificationType, NotificationSystem
from .forms import NotificationForAllUserForm


# Register your models here.
class NotificationForAllUserAdmin(BaseModelAdmin):

    fields = ['title', 'description', 'type', 'products_buy_did_not', 'products_buy', 'description_for_admin', 'sent_result']
    list_display = ['title', 'description', 'type',  get_update_at, ]
    list_filter = ['type', ]
    search_fields = ['title']
    exclude = ['from_user_obj', 'model_obj', 'notification_send_type', ]
    raw_id_fields = []
    dynamic_raw_id_fields = ['products_buy_did_not', 'products_buy']
    readonly_fields = []

    actions = None
    form = NotificationForAllUserForm

    def __init__(self, *args, **kwargs):
        Klass = NotificationForAllUserAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        Notification

    def get_queryset(self, request):

        return Notification.objects.notification_type(NotificationType.ALL_USER)

    def save_model(self, request, obj, form, change):

        set_all_user = True if not obj.id else False
        obj.save()
        if set_all_user:
            queryset = User.objects.all()

            if is_app_installed('aparnik.packages.shops.orders'):

                try:
                    from aparnik.packages.shops.orders.models import Order
                    products_buy_did_not = form.cleaned_data['products_buy_did_not']
                    if products_buy_did_not.count() > 0:
                        user_products_buy_did_not = Order.objects.get_order_products(products_buy_did_not).values_list('user', flat=True)
                        queryset = queryset.exclude(pk__in=user_products_buy_did_not)
                except:
                    pass

                try:
                    products_buy = form.cleaned_data['products_buy']
                    if products_buy.count() > 0:
                        user_products_buy = Order.objects.get_order_products(products_buy).values_list('user', flat=True)
                        queryset = queryset.filter(pk__in=user_products_buy)
                except:
                    pass

                for user in queryset:
                    m1 = Membership(user=user, notification=obj)
                    m1.save()

    def change_view(self, request, object_id, form_url='', extra_context=None):
        ''' customize edit form '''
        extra_context = extra_context or {}
        extra_context['show_save_and_continue'] = False
        extra_context['show_save'] = False
        extra_context['show_save_and_add_another'] = False  # this not works if has_add_permision is True
        return super(NotificationForAllUserAdmin, self).change_view(request, object_id, extra_context=extra_context)


class MembershipInline(BaseModelTabularInline):
    model = NotificationForSingleUser.users.through
    fields = ('user', 'is_read')
    readonly_fields = ('is_read', )
    raw_id_fields = ('user',)
    extra = 1
    exclude = []

    def __init__(self, *args, **kwargs):
        Klass = MembershipInline
        Klass_parent = BaseModelTabularInline

        super(Klass, self).__init__(*args, **kwargs)
        self.exclude = Klass_parent.exclude + self.exclude


class NotificationForSingleUserAdmin(BaseModelAdmin):

    fields = ['title', 'description', 'type', 'description_for_admin', 'sent_result']
    list_display = ['title', 'description', 'type', get_update_at,]
    list_filter = ['type', ]
    search_fields = get_user_search_fields('users') + ['title']
    exclude = ['from_user_obj', 'model_obj', 'users', 'notification_send_type', 'products_buy', 'products_buy_did_not']
    raw_id_fields = []
    dynamic_raw_id_fields = []
    readonly_fields = ()

    actions = None
    inlines = (MembershipInline,)

    def __init__(self, *args, **kwargs):
        Klass = NotificationForSingleUserAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        NotificationForSingleUser

    def get_queryset(self, request):

        return Notification.objects.notification_type(NotificationType.SINGLE_USER)

    def change_view(self, request, object_id, form_url='', extra_context=None):
        ''' customize edit form '''
        extra_context = extra_context or {}
        extra_context['show_save_and_continue'] = False
        extra_context['show_save'] = False
        extra_context['show_save_and_add_another'] = False  # this not works if has_add_permision is True
        return super(NotificationForSingleUserAdmin, self).change_view(request, object_id, extra_context=extra_context)


class NotificationSystemAdmin(BaseModelAdmin):

    fields = ['title', 'description', 'type', 'description_for_admin', 'sent_result']
    list_display = ['title', 'description', 'type', get_update_at,]
    list_filter = ['type', ]
    search_fields = get_user_search_fields('users') + ['title']
    exclude = ['from_user_obj', 'model_obj', 'users', 'notification_send_type', 'products_buy', 'products_buy_did_not']
    raw_id_fields = []
    dynamic_raw_id_fields = []
    readonly_fields = ()

    actions = None
    inlines = (MembershipInline,)

    def __init__(self, *args, **kwargs):
        Klass = NotificationSystemAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        NotificationSystem

    def get_queryset(self, request):

        return Notification.objects.notification_type(NotificationType.SYSTEM)

    def change_view(self, request, object_id, form_url='', extra_context=None):
        ''' customize edit form '''
        extra_context = extra_context or {}
        extra_context['show_save_and_continue'] = False
        extra_context['show_save'] = False
        extra_context['show_save_and_add_another'] = False  # this not works if has_add_permision is True
        return super(NotificationSystemAdmin, self).change_view(request, object_id, extra_context=extra_context)


admin.site.register(NotificationForSingleUser, NotificationForSingleUserAdmin)
admin.site.register(Notification, NotificationForAllUserAdmin)
admin.site.register(NotificationSystem, NotificationSystemAdmin)



# Generated by Django 2.2 on 2019-06-15 14:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('basemodels', '0011_auto_20190612_1035'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='basemodel',
            options={'ordering': ['-created_at'], 'verbose_name': 'مدل پایه', 'verbose_name_plural': 'مدل های پایه'},
        ),
        migrations.AlterField(
            model_name='basemodel',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, verbose_name='ساخته شده در'),
        ),
        migrations.AlterField(
            model_name='basemodel',
            name='review_average',
            field=models.FloatField(default=0.0, verbose_name='میانگین نظرات'),
        ),
        migrations.AlterField(
            model_name='basemodel',
            name='sort',
            field=models.IntegerField(default=0, verbose_name='مرتب سازی'),
        ),
        migrations.AlterField(
            model_name='basemodel',
            name='update_at',
            field=models.DateTimeField(auto_now=True, verbose_name='بروزرسانی شده در'),
        ),
        migrations.AlterField(
            model_name='basemodel',
            name='update_needed',
            field=models.BooleanField(default=False, verbose_name='نیاز به بروزرسانی'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-01-31 14:36


import datetime
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('vouchers', '0005_auto_20190130_1510'),
    ]

    operations = [
        migrations.AlterField(
            model_name='voucher',
            name='expire_at',
            field=models.DateTimeField(default=datetime.datetime(2300, 10, 5, 18, 0), verbose_name='Expire at'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-25 15:43


from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('filefields', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='SocialNetwork',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('link', models.URLField(max_length=255, verbose_name='Link')),
                ('title', models.CharField(default=None, max_length=31, verbose_name='Title')),
                ('android_app_shortcut', models.CharField(blank=True, max_length=200, null=True, verbose_name='Android shortcut')),
                ('ios_app_shortcut', models.CharField(blank=True, max_length=200, null=True, verbose_name='iOS shortcut')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('update_at', models.DateTimeField(auto_now=True, verbose_name='Update at')),
                ('icon', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='filefields.FileField', verbose_name='Icon')),
            ],
            options={
                'verbose_name': 'Social Network',
                'verbose_name_plural': 'Social Networks',
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-12-02 17:53


from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('basemodels', '0004_auto_20181103_2233'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('orders', '0002_auto_20181026_1301'),
    ]

    operations = [
        migrations.CreateModel(
            name='Voucher',
            fields=[
                ('basemodel_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='basemodels.BaseModel')),
                ('quantity', models.PositiveIntegerField(default=0, verbose_name='Quantity')),
                ('expire_at', models.DateTimeField(blank=True, null=True, verbose_name='Expire at')),
                ('is_active', models.BooleanField(default=True, verbose_name='Is Active')),
                ('is_spent', models.BooleanField(default=False, verbose_name='Is spent')),
                ('order_item_obj', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='voucher_order_item', to='orders.OrderItem', verbose_name='OrderItem')),
                ('user_obj', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='voucher_user', to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'verbose_name': 'Voucher',
                'manager_inheritance_from_future': True,
                'verbose_name_plural': 'Vouchers',
            },
            bases=('basemodels.basemodel',),
        ),
    ]



from django.conf.urls import url, include

from .views import TeacherListAPIView, TeacherModelsListAPIView

app_name = 'teachers'

urlpatterns = [
    url(r'^$', TeacherListAPIView.as_view(), name='list'),
    url(r'^(?P<teacher_id>\d+)/models/$', TeacherModelsListAPIView.as_view(), name='models-list'),
]



from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _
from django.shortcuts import get_object_or_404, get_list_or_404
from rest_framework.permissions import ( AllowAny, IsAuthenticated )
from rest_framework.response import Response
from rest_framework.status import HTTP_200_OK, HTTP_400_BAD_REQUEST
from rest_framework.generics import ListAPIView
from rest_framework.views import APIView
from rest_framework.serializers import as_serializer_error
from rest_framework.exceptions import ValidationError

from aparnik.packages.shops.orders.models import Order
from aparnik.contrib.basemodels.api.serializers import ModelDetailsPolymorphicSerializer
from ..models import Payment


class PayAPIView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request, id, format=None, *args, **kwargs):
        status = HTTP_400_BAD_REQUEST
        user = self.request.user

        content = {
            'success': False,
            'msg': '',
            'data': {
                'url': '',
            },
        }

        call_back_url = self.request.data.get('call_back_url', None)
        method = self.request.data.get('method', None)
        order = get_object_or_404(Order.objects.get_this_user(user=user), id=id)

        try:

            if not call_back_url:
                content['msg'] += 'call back is required\n'

            if not method:
                content['msg'] += 'method is required\n'

            if content['msg']:
                raise ValidationError({'__all__': [_(content['msg'])]})

            # payment = Payment.objects.request_pay(transaction=transaction, method=method, call_back_url=call_back_url, user=user)
            payment = Payment.objects.request_pay(order=order, method=method, call_back_url=call_back_url, user=user)
            content['data']['payment'] = ModelDetailsPolymorphicSerializer(payment, many=False, read_only=True, context={'request': self.request}).data
            content['success'] = True
            status = HTTP_200_OK

        except Exception as e:
            raise ValidationError(as_serializer_error(e))

        return Response(content, status=status)


class PaymentPayListAPIView(ListAPIView):
    serializer_class = ModelDetailsPolymorphicSerializer
    permission_classes = [IsAuthenticated]
    # filter_backends = (filters.SearchFilter,)

    def get_queryset(self, *args, **kwargs):
        user = self.request.user
        return Payment.objects.get_this_user(user=user)


class PaymentPayAPIView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request, uuid, format=None, *args, **kwargs):
        status = HTTP_400_BAD_REQUEST
        user = self.request.user

        content = {
            'success': False,
            'msg': '',
            'data': {
                # 'last_wallet': user.wallet,
            },
        }

        payments_obj = get_list_or_404(Payment.objects.get_this_user(user=user).order_by('-id'), uuid=uuid)

        try:
            # check wallet money
            pay_obj = payments_obj[0]
            if not (pay_obj.method == pay_obj.METHOD_WALLET and pay_obj.order_obj.status == Payment.STATUS_WAITING and pay_obj.status == Payment.STATUS_WAITING):
                raise ValidationError()


            pay_success = pay_obj.success()
            # content['data']['wallet'] = user.wallet
            content["success"] = True
            status = HTTP_200_OK

        except Exception as e:
            raise ValidationError(as_serializer_error(e))

        return Response(content, status=status)


class PaymentPayCancelAPIView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request, uuid, format=None, *args, **kwargs):
        status = HTTP_400_BAD_REQUEST
        user = self.request.user

        content = {
            'success': False,
            'msg': '',
            'data': {
            },
        }

        payments_obj = get_list_or_404(Payment.objects.get_this_user(user=user).order_by('-id'), uuid=uuid)

        try:
            # check wallet money
            pay_obj = payments_obj[0]
            pay_success = pay_obj.cancel()
            # content['data']['wallet'] = user.wallet
            content["success"] = True
            status = HTTP_200_OK

        except Exception as e:
            raise ValidationError(as_serializer_error(e))

        return Response(content, status=status)



# -*- coding: utf-8 -*-


from django.db import models
from django.utils.translation import ugettext_lazy as _

from aparnik.contrib.users.models import User
from aparnik.contrib.basemodels.models import BaseModel


# Create your models here.

class CounterManager(models.Manager):
    def get_queryset(self):
        return super(CounterManager, self).get_queryset()

    def active(self):
        return self.get_queryset()


class Counter(models.Model):
    # Action list performed by user
    # Add more actions as needed
    ACTION_VISIT = 'v'
    ACTION_SHARE = 's'
    ACTION_CHOICES = (
        (ACTION_VISIT, _('Visiting')),
        (ACTION_SHARE, _('Sharing')),
    )

    user_obj = models.ForeignKey(User, blank=True, null=True, on_delete=models.CASCADE, verbose_name=_('User'))
    model_obj = models.ForeignKey(BaseModel, on_delete=models.CASCADE, verbose_name=_('Base Model'))
    action = models.CharField(choices=ACTION_CHOICES, max_length=2, verbose_name=_('Action kind'))
    create_date = models.DateTimeField(auto_now_add=True, auto_now=False, verbose_name=_('Created at'))
    update_at = models.DateTimeField(auto_now_add=False, auto_now=True, verbose_name=_('Update at'))

    objects = CounterManager()

    def __str__(self):
        return str(self.model_obj)

    class Meta:
        verbose_name = _('Counter')
        verbose_name_plural = _('Counters')



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-01-29 16:20


import datetime
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('vouchers', '0002_auto_20181203_1533'),
    ]

    operations = [
        migrations.AlterField(
            model_name='voucher',
            name='expire_at',
            field=models.DateTimeField(default=datetime.datetime(2219, 1, 29, 16, 20, 55, 692956), verbose_name='Expire at'),
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import models
from django.utils.translation import ugettext_lazy as _

from aparnik.contrib.shortblogs.models import ShortBlog


# Create your models here.
class TermsAndConditionsManager(models.Manager):
    def get_queryset(self):
        return super(TermsAndConditionsManager, self).get_queryset()

    def active(self):
        return self.get_queryset().filter(is_active=True)


class TermsAndConditions(models.Model):
    is_active = models.BooleanField(
        verbose_name=_('Is Active'),
        default=True
    )
    short_blogs = models.ManyToManyField(ShortBlog, verbose_name=_('Content Segment'))
    created_at = models.DateTimeField(auto_now_add=True, auto_now=False, verbose_name=_('Created at'))
    update_at = models.DateTimeField(auto_now_add=False, auto_now=True, verbose_name=_('Update at'))

    objects = TermsAndConditionsManager()

    def save(self, *args, **kwargs):

        return super(TermsAndConditions, self).save(*args, **kwargs)

    def __init__(self, *args, **kwargs):
        super(TermsAndConditions, self).__init__(*args, **kwargs)

    class Meta:
        # ordering = ['latitude', 'longitude']
        verbose_name = _('Terms And Conditions')
        verbose_name_plural = _('Terms And Conditions')



# -*- coding: utf-8 -*-


from rest_framework import serializers
from aparnik.api.serializers import ModelSerializer
from rest_polymorphic.serializers import PolymorphicSerializer

from aparnik.contrib.basemodels.api.serializers import BaseModelDetailSerializer, BaseModelListSerializer
from ..models import Page


# Category List Serializer
class PageListSerializer(BaseModelListSerializer):

    class Meta:
        model = Page
        fields = BaseModelListSerializer.Meta.fields + [
            'title',
            'english_title',
        ]


# Category Details Serializer
class PageDetailsSerializer(PageListSerializer, BaseModelDetailSerializer):
    content = serializers.SerializerMethodField()

    class Meta:
        model = Page
        fields = PageListSerializer.Meta.fields + BaseModelDetailSerializer.Meta.fields + [
            'content',
        ]

    def get_content(self, obj):
        from aparnik.contrib.segments.models import PageSort
        from aparnik.contrib.segments.api.serializers import PageSortListSerializer
        return PageSortListSerializer(PageSort.objects.filter(page_obj=obj), many=True, read_only=True,
                                                   context=self.context).data


# Category List Polymorphic
class PageListPolymorphicSerializer(PolymorphicSerializer):
    def __init__(self, *args, **kwargs):
        self.model_serializer_mapping = {
            Page: PageListSerializer,
        }
        super(PageListPolymorphicSerializer, self).__init__(*args, **kwargs)

    model_serializer_mapping = {}


# Category Details Polymorphic
class PageDetailsPolymorphicSerializer(PolymorphicSerializer):
    def __init__(self, *args, **kwargs):
        self.model_serializer_mapping = {
            Page: PageDetailsSerializer,
        }
        super(PageDetailsPolymorphicSerializer, self).__init__(*args, **kwargs)

    model_serializer_mapping = {}



# -*- coding: utf-8 -*-


from django.db import models
from django.db.models.signals import post_save
from django.contrib.auth.models import AbstractBaseUser, BaseUserManager, PermissionsMixin
from django.urls import reverse
from django.core.validators import RegexValidator, ValidationError, MinValueValidator, MaxValueValidator
from django.utils.timezone import now
from django.utils.translation import ugettext_lazy as _

from django_enumfield import enum

from aparnik.settings import aparnik_settings, Setting
from aparnik.utils.formattings import formatprice
from aparnik.utils.fields import PhoneField, NationalCodeField, PriceField

from aparnik.contrib.province.models import City

import uuid
from kavenegar import *


class UserManager(BaseUserManager):
    # use_in_migrations = True
    def get_queryset(self):
        return super(UserManager, self).get_queryset()

    def active(self):
        return self.get_queryset()

    # python manage.py createsuperuser
    def create_superuser(self, username, first_name, last_name, is_staff=None, password=None):

        user = self.model(
            username=username,
            first_name=first_name,
            last_name=last_name,
            sex=User.SEX_MALE,
            is_staff=True,
            is_active=True,
            is_superuser=True
        )
        user.save()
        return user

    def admins(self):
        return self.active().filter(groups__id__in=[1])


# PermissionsMixin
class User(AbstractBaseUser, PermissionsMixin):
    SEX_MALE = 'M' # TYPE_INT = 0
    SEX_WOMAN = 'F'
    SEX_TYPE = (
        (SEX_MALE, "مرد"),
        (SEX_WOMAN, "زن")
    )
    sys_id = models.AutoField(primary_key=True, blank=True, verbose_name=_('System ID'))
    uuid = models.UUIDField(default=uuid.uuid4, editable=False, verbose_name=_('UUID'))
    password = None
    passwd = models.CharField(max_length=255, blank=True, null=True, verbose_name=_('Password'))
    username = PhoneField(max_length=30, mobile=True, unique=True, verbose_name=_('Mobile'))
    email = models.EmailField(null=True, blank=True, verbose_name=_('Email'))
    first_name = models.CharField(max_length=12, null=True, blank=True, verbose_name=_('Name'))
    last_name = models.CharField(max_length=30, null=True, blank=True, verbose_name=_('Family'))
    username_mention = models.CharField(default='a', max_length=100, unique=True, validators=[RegexValidator(
            regex='^[a-zA-Z_\-0-9]+$',
            message='Username must be Alphanumeric',
            code='invalid_username'
        ),], verbose_name=_('Username for mention'))
    sex = models.CharField(max_length=1, choices=SEX_TYPE, default='M', verbose_name=_('Sex'))
    avatar = models.ForeignKey('filefields.FileField', null=True, blank=True, on_delete=models.CASCADE, verbose_name=_('Avatar'))
    current_city = models.ForeignKey(City, on_delete=models.CASCADE, blank=True, null=True, verbose_name=_('Current City'))
    birthdate = models.DateTimeField(blank=True, null=True, verbose_name=_('Birthdate'))
    is_staff = models.BooleanField(default=False, verbose_name=_('Is Staff'))
    is_active = models.BooleanField(default=True, verbose_name=_('Is Active'))
    token = models.CharField(blank=True, null=True, max_length=4, verbose_name=_('Token'))
    token_time = models.DateTimeField(blank=True, null=True, verbose_name=_('Token Time'))
    limit_device_login = models.IntegerField(default=-1, help_text=_('If the number is set to -1, then the value in the settings will apply'), verbose_name=_('Limit device login'))
    invitation_code = models.CharField(editable=False, null=True, blank=True, max_length=36, verbose_name=_('Invitation Code'))
    verify_mobile = models.BooleanField(default=False, verbose_name=_('Verify Mobile'))
    wallet = PriceField(big=True, default=0, verbose_name=_('Wallet'))
    co_sale_percentage_value = models.IntegerField(default=-1, validators=[MaxValueValidator(100), MinValueValidator(-1)], help_text=_('If the number is set to -1, then the value in the settings will apply'), verbose_name=_('Co Sale Percentage'))
    company_national_number = models.CharField(max_length=255, null=True, blank=True, verbose_name=_('Company national number'))
    company_registration_number = models.CharField(max_length=255, null=True, blank=True, verbose_name=_('Company registeration number'))
    company_economical_number = models.CharField(max_length=255, null=True, blank=True, verbose_name=_('Company economical number'))

    created_at = models.DateTimeField(auto_now_add=True, auto_now=False, verbose_name=_('Created at'))
    update_at = models.DateTimeField(auto_now_add=False, auto_now=True, verbose_name=_('Update at'))

    objects = UserManager()

    USERNAME_FIELD = "username"
    # REQUIRED_FIELDS must contain all required fields on your User model,
    # but should not contain the USERNAME_FIELD or password as these fields will always be prompted for.
    REQUIRED_FIELDS = ['first_name', 'last_name', 'is_staff']

    class Meta:
        app_label = 'aparnik_users'
        verbose_name = _('User')
        verbose_name_plural = _('Users')

    def __str__(self):
        return '%s %s' % (self.username, self.get_full_name())

    def clean(self):
        if aparnik_settings.USER_LOGIN_WITH_PASSWORD and self.passwd == '':
            raise ValidationError({'passwd': [_('Required password, Please Disabel USER_LOGIN_WITH_PASSWORD')]})

        if self.username_mention == 'a':
            from faker import Faker
            faker = Faker()
            username = faker.user_name()
            while User.objects.filter(username_mention=username).exists():
                username = faker.user_name()
            self.username_mention = username

    def save(self, *args, **kwargs):

        def generate():
            base = 4
            invitation_code = uuid.uuid4()

            if aparnik_settings.USER_INVITATION_CODE_INT:

                invitation_code = invitation_code.int
            else:
                invitation_code = invitation_code.__str__().replace('-', '')

            invitation_code = invitation_code[base:base + aparnik_settings.USER_INVITATION_CODE_LENGTH]
            if User.objects.filter(invitation_code=invitation_code).count() > 0:
                return generate()
            return invitation_code

        if self.invitation_code is None:
            self.invitation_code = generate()

        self.full_clean()
        return super(User, self).save(*args, **kwargs)

    def get_api_uri(self):
        return reverse("aparnik-api:users:detail", kwargs={"username": self.username})

    def get_full_name(self):
        return "%s %s" % (self.first_name, self.last_name)

    def get_short_name(self):
        return self.first_name

    def device_login_count(self):
        return self.devicelogin_set.get_login_count(user=self)

    def is_can_login(self, device_id=None):

        limit_device_login = self.limit_device_login if self.limit_device_login >= 0 else Setting.objects.get(key='USER_DEFAULT_LIMIT_DEVICE_LOGIN').get_value()

        if Setting.objects.get(key='USER_IS_LIMIT_DEVICE_LOGIN_ACTIVE').get_value():

            if device_id and self.devicelogin_set.active().filter(device_id=device_id).exists():
                return True

            if device_id and Setting.objects.get(key='USER_IS_LIMIT_DEVICE_LOGIN_ACTIVE').get_value() and self.devicelogin_set.get_login_count(user=self) >= limit_device_login :
                raise ValidationError(_('تعداد دستگاه های فعال شما بیش از %(value)s است.'), params={'value': limit_device_login})

        return True

    def reset_password(self):
        passwd = str(uuid.uuid4().int)[:5]

        if aparnik_settings.VERIFY_FAKE:

            passwd = aparnik_settings.VERIFY_FAKE_CODE

        self.passwd = passwd
        self.save()

        if aparnik_settings.SEND_SMS:
            try:
                api = KavenegarAPI(aparnik_settings.SMS_API_KEY)
                params = {
                    'receptor': self.username,
                    'template': aparnik_settings.SMS_OTA_NAME,
                    'token': self.first_name.replace(' ', '-') if self.first_name is not None else 'مهمان',
                    'token2': self.passwd,
                    'type': 'sms',  # sms vs call
                }
                response = api.verify_lookup(params)
                # print(response)

            except APIException as e:
                print(e)

            except HTTPException as e:
                print(e)

    def OTAVerify(self, token):

        if aparnik_settings.VERIFY_FAKE:

            if token == aparnik_settings.VERIFY_FAKE_CODE:
                self.token = None
                self.token_time = None
                return True

        if self.token is None:
            return False

        diff = (now() - self.token_time).total_seconds()

        if (diff < int(aparnik_settings.SMS_EXPIRE_TOKEN) and diff > 0 ) and self.token == token:

            self.token = None
            self.token_time = None
            self.save()
            return True

        return False

    def OTARequest(self, app_signature=None):

        if self.token_time and (now() - self.token_time).total_seconds() < int(aparnik_settings.SMS_EXPIRE_TOKEN):
            # raise ValidationError({'username': [_('Verification code sent before. Please wait until code expired.')]})
            return

        self.token = str(uuid.uuid4().int)[-4:]
        self.token_time = now()
        self.save()

        if aparnik_settings.SEND_SMS:
            try:
                if not app_signature:
                    app_signature = 'Code:%s' % self.token

                api = KavenegarAPI(aparnik_settings.SMS_API_KEY)
                params = {
                    'receptor': self.username,
                    'template': aparnik_settings.SMS_OTA_NAME,
                    'token': self.token, #self.first_name.replace(' ', '-') if self.first_name is not None else 'مهمان',
                    'token2': app_signature,
                    'type': 'sms',  # sms vs call
                }
                response = api.verify_lookup(params)
                # print(response)

            except APIException as e:
                print(e)

            except HTTPException as e:
                print(e)

    # # this methods are require to login super user from admin panel
    # def has_perm(self, perm, obj=None):
    #     return self.is_staff
    #
    # # this methods are require to login super user from admin panel
    # def has_module_perms(self, app_label):
    #     return self.is_staff
    @property
    def is_new_registered(self):
        new_register = (not self.first_name) or (not self.last_name)
        if aparnik_settings.USER_LOGIN_WITH_PASSWORD:
            new_register = new_register or not self.verify_mobile
        return new_register

    @property
    def wallet_string(self):
        return '%s' % formatprice.format_price(self.wallet)

    @property
    def is_admin(self):
        return self.groups.filter(name='admin').exists() or self.is_superuser

    @property
    def co_sale_percentage(self):
        if self.co_sale_percentage_value == -1:
            from aparnik.settings import Setting
            inviter_per_purchase_value = Setting.objects.get(key='INVITER_GIFT_CREDITS_PER_PURCHASE').get_value()
            return inviter_per_purchase_value
        return self.co_sale_percentage_value

    @property
    def co_sale_summary(self):
        from aparnik.packages.shops.cosales.models import CoSale
        return CoSale.objects.summary(user=self)

    # دعوت شده توسط
    @property
    def invited_by(self):
        invited_by = self.invite.first()
        if invited_by:
            return invited_by.invited_by
        return None


def post_save_user_receiver(sender, instance, created, *args, **kwargs):

    if created:
        from aparnik.packages.shops.products.models import Product
        from aparnik.packages.shops.orders.models import Order
        from aparnik.packages.shops.vouchers.models import Voucher
        number_gift = round(Setting.objects.get(key='USER_REGISTER_GIFT_BON_CREDITS').get_value() / Setting.objects.get(key='APARNIK_BON_VALUE').get_value())
        manager_product = Product.objects.get(pk=Setting.objects.get(key='MANAGER_PRODUCT_ID').get_value())
        order = Order.objects.create(user=instance)
        order.add_item(manager_product)
        order.save()
        order.pay_success(status=Order.STATUS_COMPLETE)
        order.save()
        voucher = Voucher.objects.get(order_item_obj=order.items.first(), user_obj=instance)
        voucher.quantity = number_gift
        voucher.quantity_remain = number_gift
        voucher.save()
    if created and instance.is_new_registered:
        instance.OTARequest()


post_save.connect(post_save_user_receiver, sender=User)


class DeviceLoginManager(models.Manager):

    def get_queryset(self):
        return super(DeviceLoginManager, self).get_queryset()

    def active(self):
        return self.get_queryset().filter(is_active=True)

    def get_login(self, user):
        return self.active().filter(user=user).values_list('device_id', flat=True).distinct()

    def get_login_count(self, user):
        return self.get_login(user=user).count()


# Create your models here.
class DeviceType(enum.Enum):
    ANDROID = 0
    IOS = 1
    WEB = 2

    labels = {
        ANDROID: _('Android'),
        IOS: _('iOS'),
        WEB: _('Web')
    }

    @staticmethod
    def find(value):
        switcher = {
            'a': DeviceType.ANDROID,
            'i': DeviceType.IOS,
            'w': DeviceType.WEB
        }

        return switcher.get(value, None)

    @staticmethod
    def value(value):
        switcher = {
            DeviceType.ANDROID: 'android',
            DeviceType.IOS: 'ios',
            DeviceType.WEB: 'web',
        }

        return switcher.get(value, None)


class DeviceLogin(models.Model):

    user = models.ForeignKey('aparnik_users.User', on_delete=models.CASCADE, verbose_name=_('User'))
    version_number = models.FloatField(verbose_name=_('Version Number'))
    device_id = models.CharField(max_length=255, verbose_name=_('Device Id'))
    os_version = models.CharField(max_length=255, verbose_name=_('OS Version'))
    device_model = models.CharField(max_length=255, verbose_name=_('Device Model'))
    device_type = enum.EnumField(DeviceType, default=DeviceType.ANDROID, verbose_name=_('Device Type'))
    is_active = models.BooleanField(default=True, verbose_name=_('Is Active'))
    created_at = models.DateTimeField(auto_now_add=True, auto_now=False)
    update_at = models.DateTimeField(auto_now_add=False, auto_now=True)

    objects = DeviceLoginManager()

    def __str__(self):
        return self.device_id

    class Meta:
        verbose_name = _('Device Login')
        verbose_name_plural = _('Devices Login')



# Generated by Django 2.2 on 2019-06-15 14:30

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('segments', '0012_auto_20181214_1330'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='basesegment',
            options={'verbose_name': 'بخش پایه', 'verbose_name_plural': 'بخش پایه'},
        ),
        migrations.AlterField(
            model_name='basesegment',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, verbose_name='ساخته شده در'),
        ),
        migrations.AlterField(
            model_name='basesegment',
            name='is_active',
            field=models.BooleanField(default=True, verbose_name='فعال؟'),
        ),
        migrations.AlterField(
            model_name='basesegment',
            name='model_obj',
            field=models.ManyToManyField(through='segments.SegmentSort', to='basemodels.BaseModel', verbose_name='چه چیز را نمایش دهد؟'),
        ),
        migrations.AlterField(
            model_name='basesegment',
            name='pages',
            field=models.ManyToManyField(blank=True, related_name='segment_pages', through='segments.PageSort', to='pages.Page', verbose_name='صفحه'),
        ),
        migrations.AlterField(
            model_name='basesegment',
            name='title',
            field=models.CharField(max_length=60, verbose_name='عنوان'),
        ),
        migrations.AlterField(
            model_name='basesegment',
            name='update_at',
            field=models.DateTimeField(auto_now=True, verbose_name='بروزرسانی شده در'),
        ),
        migrations.AlterField(
            model_name='pagesort',
            name='page_obj',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='page_sort_page', to='pages.Page', verbose_name='صفحه'),
        ),
        migrations.AlterField(
            model_name='pagesort',
            name='sort',
            field=models.IntegerField(default=0, verbose_name='مرتب سازی'),
        ),
        migrations.AlterField(
            model_name='segmentsort',
            name='model_obj',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='segment_sort_model', to='basemodels.BaseModel', verbose_name='چه چیز را نمایش دهد؟'),
        ),
        migrations.AlterField(
            model_name='segmentsort',
            name='sort',
            field=models.IntegerField(default=0, verbose_name='مرتب سازی'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-06-13 18:14


from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('basemodels', '0011_auto_20190612_1035'),
        ('reviews', '0004_reviewsummary_percentage'),
    ]

    operations = [
        migrations.AddField(
            model_name='review',
            name='model_obj2',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='reviews_set', to='basemodels.BaseModel', verbose_name='Model'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-01-21 09:49


from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0008_alter_user_username_max_length'),
        ('managements', '0003_auto_20181126_1322'),
    ]

    operations = [
        migrations.AddField(
            model_name='fieldlist',
            name='group',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.CASCADE, to='auth.Group', verbose_name='group'),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-04-23 11:06


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aparnik_users', '0021_auto_20190423_1058'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='company_economical_number',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='Company economical number'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-12-09 21:39


from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('province', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='city',
            options={'verbose_name': 'City', 'verbose_name_plural': 'Cities'},
        ),
        migrations.AlterModelOptions(
            name='province',
            options={'verbose_name': 'Province', 'verbose_name_plural': 'Provinces'},
        ),
        migrations.AlterModelOptions(
            name='shahrak',
            options={'verbose_name': 'Shahrak', 'verbose_name_plural': 'Shahraks'},
        ),
        migrations.AlterModelOptions(
            name='town',
            options={'verbose_name': 'Town', 'verbose_name_plural': 'Towns'},
        ),
        migrations.AlterField(
            model_name='city',
            name='province',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='province.Province', verbose_name='Province'),
        ),
        migrations.AlterField(
            model_name='city',
            name='title',
            field=models.CharField(max_length=255, verbose_name='Title'),
        ),
        migrations.AlterField(
            model_name='province',
            name='title',
            field=models.CharField(max_length=100, verbose_name='Title'),
        ),
        migrations.AlterField(
            model_name='shahrak',
            name='province',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='province.Province', verbose_name='Province'),
        ),
        migrations.AlterField(
            model_name='shahrak',
            name='title',
            field=models.TextField(verbose_name='Title'),
        ),
        migrations.AlterField(
            model_name='town',
            name='city',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='province.City', verbose_name='City'),
        ),
        migrations.AlterField(
            model_name='town',
            name='title',
            field=models.CharField(max_length=255, verbose_name='Title'),
        ),
    ]



from django.conf.urls import url, include
from rest_framework_jwt.views import verify_jwt_token

from aparnik.settings import aparnik_settings
from aparnik.contrib.users.api.views import UserSendSMSAPI, UserVerifySMSAPI, FCMAddTokenAPIView, UserLogoutAPIView, \
    UserTokenAPIView, UserForgetPasswordAPI, UserSubsetListAPIView
UserLoginAPI = aparnik_settings.USER_LOGIN_API_VIEW
UserListAPIView = aparnik_settings.USER_LIST_API_VIEW
UserDetailAPIView = aparnik_settings.USER_DETAIL_API_VIEW
UserCreateAPIView = aparnik_settings.USER_CREATE_API_VIEW
UserUpdateAPIView = aparnik_settings.USER_UPDATE_API_VIEW

app_name = 'aparnik_users'

urlpatterns = [
    url(r'^$', UserListAPIView.as_view(), name='list'),
    url(r'^subset/$', UserSubsetListAPIView.as_view(), name='subset'),
    url(r'^login$', UserLoginAPI.as_view(), name='login'),
    url(r'^forget-password/', UserForgetPasswordAPI.as_view(), name='forget-password'),
    url(r'^logout/', UserLogoutAPIView.as_view(), name='logout'),
    url(r'^send-sms/', UserSendSMSAPI.as_view(), name='send-sms'),
    url(r'^verify-sms/', UserVerifySMSAPI.as_view(), name='verify-sms'),
    url(r'^notification-add-token/', FCMAddTokenAPIView.as_view(), name='FCM-add-token'),
    url(r'^create', UserCreateAPIView.as_view(), name='create'),
    url(r'^token/verify', verify_jwt_token, name='verify'),
    url(r'^token', UserTokenAPIView.as_view(), name='token'),
    url(r'^(?P<username>[\w-]+)/$', UserDetailAPIView.as_view(), name='detail'),
    url(r'^(?P<username>[\w-]+)/update$', UserUpdateAPIView.as_view(), name='update'),

]


# -*- coding: utf-8 -*-


from django.db import models
from django.contrib.auth import get_user_model
from django.utils.translation import ugettext_lazy as _

from aparnik.utils.fields import *
from aparnik.contrib.basemodels.models import BaseModel, BaseModelManager

User = get_user_model()


# Create your models here.
class BankNameManager(BaseModelManager):

    def get_queryset(self):
        return super(BankNameManager, self).get_queryset()

    def active(self):
        return super(BankNameManager, self).active().filter(is_active=True)


class BankName(BaseModel):

    title = models.CharField(max_length=255, verbose_name=_('Title'))

    is_active = models.BooleanField(default=True, verbose_name=_('Is Active'))

    objects = BankNameManager()

    def clean(self):
        pass

    def save(self, *args, **kwargs):
        self.full_clean()
        return super(BankName, self).save(*args, **kwargs)

    def __str__(self):
        return self.title

    class Meta:
        verbose_name = _('Bank Name')
        verbose_name_plural = _('Bank Names')


class BankAccountManager(BaseModelManager):

    def get_queryset(self):
        return super(BankAccountManager, self).get_queryset()

    def active(self):
        return super(BankAccountManager, self).active().filter(is_active=True)

    def this_user(self, user):
        if not user.is_authenticated:
            return BankAccount.objects.none()
        return self.active().filter(user_obj=user)


class BankAccount(BaseModel):

    user_obj = models.ForeignKey(User, related_name='bankaccounts', on_delete=models.CASCADE, verbose_name=_('User'))
    bank_name_obj = models.ForeignKey(BankName, related_name='bankaccounts_name', on_delete=models.CASCADE, verbose_name=_('Bank Name'))
    account_number = models.CharField(max_length=255, verbose_name=_('Account Number'))
    card_number = models.CharField(max_length=16, verbose_name=_('Card Number'))
    shaba_number = models.CharField(max_length=255, unique=True, verbose_name=_('Shaba Number'))
    is_default = models.BooleanField(default=False, verbose_name=_('Is default'))
    is_active = models.BooleanField(default=True, verbose_name=_('Is Active'))

    objects = BankAccountManager()

    def save(self, *args, **kwargs):
        self.full_clean()
        if BankAccount.objects.active().filter(user_obj=self.user_obj, is_default=True).count() == 0:
            self.is_default = True
        return super(BankAccount, self).save(*args, **kwargs)

    def __str__(self):
        return str(self.user_obj)

    class Meta:
        verbose_name = _('Bank Account')
        verbose_name_plural = _('Bank Accounts')

    def set_is_default(self, user):
        if self.user_obj == user:
            BankAccount.objects.this_user(user=user).update(is_default=False)
            self.is_default = True
            self.save()
            return True
        return False



# -*- coding: utf-8 -*-


from rest_framework.generics import RetrieveAPIView, CreateAPIView, UpdateAPIView, ListAPIView
from rest_framework.permissions import (AllowAny, IsAuthenticated)

from ..models import Book, Publisher, WriterTranslator
from .serializers import PublisherListSerializer, PublisherDetailsSerializer, WriterTranslatorListSerializer, WriterTranslatorDetailsSerializer
from aparnik.packages.shops.products.api.serializers import ProductListPolymorphicSerializer


class BookListAPIView(ListAPIView):
    serializer_class = ProductListPolymorphicSerializer
    permission_classes = [AllowAny]
    queryset = Book.objects.all()
    # filter_backends = (filters.SearchFilter,)
    search_fields = ('id',)


class BookUserListAPIView(ListAPIView):
    serializer_class = ProductListPolymorphicSerializer
    permission_classes = [IsAuthenticated]
    # filter_backends = (filters.SearchFilter,)
    search_fields = ('id',)

    def get_queryset(self):
        user = self.request.user
        from aparnik.packages.shops.orders.models import Order
        queryset = Book.objects.filter(
            id__in=Order.objects.get_this_user_bought(user=user, custom_type=Book).values_list('items__product_obj', flat=True))
        return queryset


class PublisherListAPIView(ListAPIView):
    serializer_class = PublisherListSerializer
    permission_classes = [AllowAny]
    queryset = Publisher.objects.all()
    # filter_backends = (filters.SearchFilter,)
    search_fields = ('id',)


class PublisherDetailsAPIView(RetrieveAPIView):
    serializer_class = PublisherDetailsSerializer
    queryset = Publisher.objects.all()
    permission_classes = [AllowAny]
    lookup_url_kwarg = 'publisher_id'
    lookup_field = 'id'


class WriterTranslatorListAPIView(ListAPIView):
    serializer_class = WriterTranslatorListSerializer
    permission_classes = [AllowAny]
    queryset = WriterTranslator.objects.all()
    # filter_backends = (filters.SearchFilter,)
    search_fields = ('id',)


class WriterTranslatorDetailAPIView(RetrieveAPIView):
    serializer_class = WriterTranslatorDetailsSerializer
    queryset = WriterTranslator.objects.all()
    permission_classes = [AllowAny]
    lookup_url_kwarg = 'library_id'
    lookup_field = 'id'



# Generated by Django 2.2 on 2019-06-15 14:30

import ckeditor_uploader.fields
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('courses', '0019_auto_20190526_1043'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='basecourse',
            options={'verbose_name': 'دوره پایه', 'verbose_name_plural': 'دوره های پایه'},
        ),
        migrations.AlterModelOptions(
            name='course',
            options={'verbose_name': 'دوره', 'verbose_name_plural': 'دوره ها'},
        ),
        migrations.AlterModelOptions(
            name='coursefile',
            options={'verbose_name': 'فایل', 'verbose_name_plural': 'فایل ها'},
        ),
        migrations.AlterModelOptions(
            name='coursesegment',
            options={'verbose_name': 'بخش دوره', 'verbose_name_plural': 'بخش دوره ها'},
        ),
        migrations.AlterModelOptions(
            name='coursesummary',
            options={'ordering': ('id',), 'verbose_name': 'خلاصه دوره', 'verbose_name_plural': 'خلاصه دوره'},
        ),
        migrations.AlterField(
            model_name='basecourse',
            name='description',
            field=ckeditor_uploader.fields.RichTextUploadingField(blank=True, null=True, verbose_name='توضیحات'),
        ),
        migrations.AlterField(
            model_name='course',
            name='banner',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='course_banner', to='filefields.FileField', verbose_name='بنر'),
        ),
        migrations.AlterField(
            model_name='course',
            name='category_obj',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='categories.Category', verbose_name='دسته بندی'),
        ),
        migrations.AlterField(
            model_name='course',
            name='cover',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='course_cover', to='filefields.FileField', verbose_name='تصویر جلد'),
        ),
        migrations.AlterField(
            model_name='course',
            name='dependency_obj',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='dependency', to='courses.Course', verbose_name='وابستگی'),
        ),
        migrations.AlterField(
            model_name='course',
            name='parent_obj',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='courses.Course', verbose_name='والد'),
        ),
        migrations.AlterField(
            model_name='course',
            name='publish_date',
            field=models.DateTimeField(default=django.utils.timezone.now, verbose_name='تاریخ انتشار'),
        ),
        migrations.AlterField(
            model_name='course',
            name='publish_day',
            field=models.CharField(blank=True, choices=[('1', 'شنبه'), ('2', 'یکشنبه'), ('3', 'دوشنبه'), ('4', 'سه شنبه'), ('5', 'چهارشنبه'), ('6', 'پنج شنبه'), ('7', 'جمعه')], max_length=1, null=True, verbose_name='روز'),
        ),
        migrations.AlterField(
            model_name='course',
            name='publish_month',
            field=models.CharField(blank=True, choices=[('1', 'فروردین'), ('2', 'اردیبهشت'), ('3', 'خرداد'), ('4', 'تیر'), ('5', 'مرداد'), ('6', 'شهریور'), ('7', 'مهر'), ('8', 'آبان'), ('9', 'آذر'), ('10', 'دی'), ('11', 'بهمن'), ('12', 'اسفند')], max_length=2, null=True, verbose_name='ماه'),
        ),
        migrations.AlterField(
            model_name='course',
            name='publish_week',
            field=models.CharField(blank=True, choices=[('1', '1'), ('2', '2'), ('3', '3'), ('4', '4')], max_length=1, null=True, verbose_name='هفته'),
        ),
        migrations.AlterField(
            model_name='course',
            name='teacher_obj',
            field=models.ManyToManyField(blank=True, to='teachers.Teacher', verbose_name='مدرسین'),
        ),
        migrations.AlterField(
            model_name='coursefile',
            name='course_obj',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='courses.Course', verbose_name='دوره'),
        ),
        migrations.AlterField(
            model_name='coursefile',
            name='seconds',
            field=models.BigIntegerField(default=0, verbose_name='زمان'),
        ),
        migrations.AlterField(
            model_name='coursesummary',
            name='course',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='total_times', to='courses.BaseCourse', verbose_name='دوره'),
        ),
        migrations.AlterField(
            model_name='coursesummary',
            name='file_count',
            field=models.PositiveIntegerField(default=0, verbose_name='تعداد فایل ها'),
        ),
        migrations.AlterField(
            model_name='coursesummary',
            name='file_count_preview',
            field=models.PositiveIntegerField(default=0, verbose_name='تعداد فایل های پیش نمایش'),
        ),
        migrations.AlterField(
            model_name='coursesummary',
            name='total_time_seconds',
            field=models.BigIntegerField(default=0, verbose_name='زمان'),
        ),
        migrations.AlterField(
            model_name='coursesummary',
            name='type',
            field=models.CharField(choices=[('M', 'فیلم'), ('V', 'صدا'), ('P', 'پی دی اف'), ('I', 'عکس')], max_length=1, verbose_name='نوع'),
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import models
from django.db.models.signals import post_save
from django.db.models import Avg
from django.contrib.auth import get_user_model
from django.core.validators import ValidationError
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _
from django.db.models.signals import post_save
from django.utils.text import Truncator

from aparnik.settings import Setting
from aparnik.contrib.segments.models import BaseSegment, BaseSegmentManager
from aparnik.contrib.basemodels.models import BaseModel, BaseModelManager
from aparnik.packages.educations.courses.models import Course
from aparnik.contrib.notifications.models import Notification

User = get_user_model()


# Total Time Manager
class ReviewSummaryManager(models.Manager):

    def get_queryset(self):
        return super(ReviewSummaryManager, self).get_queryset()


class ReviewSummary(models.Model):
    rate = models.CharField(max_length=1, verbose_name=_('Rate'))
    count = models.BigIntegerField(default=0, verbose_name=_('Count'))
    percentage = models.FloatField(verbose_name=_('Percentage'))
    model = models.ForeignKey(BaseModel, related_name='review_summaries', on_delete=models.CASCADE, verbose_name=_('Models'))

    objects = ReviewSummaryManager()

    class Meta:
        verbose_name = _('Reivew Summary')
        verbose_name_plural = _('Review Summary')

    def __str__(self):
        return '%s' % self.id


# Review Manager
class BaseReviewManager(BaseModelManager):
    def get_queryset(self):
        return super(BaseReviewManager, self).get_queryset()

    def get_this_user(self, user):
        if not user.is_authenticated:
            return BaseReview.objects.none()
        return self.get_queryset().filter(user_obj=user)

    def active(self, user=None):
        return super(BaseReviewManager, self).active(user).filter(is_published=True)

    def model_base_reviews(self, model_obj, user_obj=None, with_children=False):
        if not isinstance(model_obj, BaseModel):
            model_obj = BaseModel.objects.get(pk=model_obj)

        objs = self.active().filter(model_obj=model_obj)
        if not with_children:
            objs = objs.filter(parent_obj__isnull=True)
        if user_obj:
            query = objs | self.get_queryset().filter(model_obj=model_obj, parent_obj__isnull=True)
            if model_obj.has_permit_full_access(user_obj):
                return query
            elif isinstance(model_obj, Course):
                if model_obj.teachers.all().filter(user_obj=user_obj).exists():
                    # Teacher mode
                    return query
            # get user base review add
            return objs | self.get_queryset().filter(model_obj=model_obj, parent_obj__isnull=True,
                                                     user_obj=user_obj, is_published=False)
        return objs


# Review Model
class BaseReview(BaseModel):
    title = models.CharField(max_length=255, verbose_name=_('Title'))
    description = models.TextField(blank=True, verbose_name=_('Description'))
    is_published = models.BooleanField(default=False, verbose_name=_('Published'))
    user_obj = models.ForeignKey(User, on_delete=models.CASCADE, verbose_name=_('User'))
    parent_obj = models.ForeignKey('self', related_name='child', on_delete=models.CASCADE, blank=True, null=True, verbose_name=_('Parent'))

    objects = BaseReviewManager()

    def __str__(self):
        return self.title

    def __init__(self, *args, **kwargs):
        super(BaseReview, self).__init__(*args, **kwargs)

    def clean(self):

        if self.id:
            obj = BaseReview.objects.get(id=self.id)
            if self.is_published != obj.is_published:
                pass
                # TODO: check permission

    def save(self, *args, **kwargs):
        self.full_clean()
        if not self.id:
            self.update_needed = True
        return super(BaseReview, self).save(*args, **kwargs)

    def get_api_like_uri(self):
        return reverse("aparnik-api:reviews:like:list", kwargs={"review_id": self.id})

    def get_api_dislike_uri(self):
        return reverse("aparnik-api:reviews:dislike:list", kwargs={"review_id": self.id})

    def get_api_action_like(self):
        return reverse("aparnik-api:reviews:like:set", kwargs={"review_id": self.id})

    def get_api_action_dislike(self):
        return reverse("aparnik-api:reviews:dislike:set", kwargs={"review_id": self.id})

    def get_api_action_base_review_status_set(self):
        return reverse("aparnik-api:reviews:status-set", kwargs={"review_id": self.id})

    class Meta:
        verbose_name = _('Base Review')
        verbose_name_plural = _('Base Reviews')

    def has_permit_replay(self, user):
        if not user.is_authenticated:
            return False
        if self.has_permit_full_access(user):
            return True
        # اگر پدر نداشت مجوز پاسخ را دارد. باید سطح دسترسی فقط کنترل شود.
        return self.parent_obj is None

    def has_permit_change_status(self, user):
        if not user.is_authenticated:
            return False
        if self.has_permit_full_access(user):
            return True
        return self.model_obj.get_real_instance().has_permit_edit(user=user)

    def has_permit_edit(self, user):
        if not user.is_authenticated:
            return False
        if self.has_permit_full_access(user):
            return True
        return self.model_obj.get_real_instance().has_permit_edit(user=user)

    def is_like(self, user):
        return self.like_set.filter(user_obj=user, is_like=True).exists()

    def is_dislike(self, user):
        return self.like_set.filter(user_obj=user, is_like=False).exists()


# Review Manager
class ReviewManager(BaseReviewManager):

    def get_queryset(self):
        return super(ReviewManager, self).get_queryset()

    def active(self, user=None):
        return super(ReviewManager, self).active(user)

    def get_this_user(self, user):
        return super(ReviewManager, self).get_this_user(user=user)

    def model_reviews(self, model_obj, user_obj=None, with_children=False):
        return super(ReviewManager, self).model_base_reviews(model_obj, user_obj, with_children)

    def model_review_count(self, model_obj, user_obj=None):
        return self.model_reviews(model_obj, user_obj, with_children=True).count()

    def model_review_avg(self, model_obj):
        return self.model_reviews(model_obj).aggregate(review_average=Avg('rate'))['review_average']


# Review Model
class Review(BaseReview):
    rate = models.FloatField(verbose_name=_('Rate'))
    # ابجکتی که نظر براش ثبت شده به عنوان مثال کالای اول
    model_obj = models.ForeignKey(BaseModel, related_name='reviews_set', on_delete=models.CASCADE, verbose_name=_('Model'))

    objects = ReviewManager()

    def __str__(self):
        return '{} {}'.format(self.title, self.rate)

    class Meta:
        verbose_name = _('Review')
        verbose_name_plural = _('Reviews')


# LIKE MANAGER
class LikeManager(models.Manager):

    def get_queryset(self):
        return super(LikeManager, self).get_queryset()

    def get_this_user(self, user):
        if not user.is_authenticated:
            return Like.objects.none()
        return self.get_queryset().filter(user=user)

    def review(self, review_obj):
        return self.get_queryset().filter(review_obj=review_obj)

    def like(self, review_obj):
        return self.review(review_obj=review_obj).filter(is_like=True)

    def dislike(self, review_obj):
        return self.review(review_obj=review_obj).filter(is_like=False)

    def likes_count(self, review_obj):
        return self.like(review_obj=review_obj).count()

    def dislikes_count(self, review_obj):
        return self.dislike(review_obj=review_obj).count()


# LIKE MODEL
class Like(models.Model):
    review_obj = models.ForeignKey(BaseReview, on_delete=models.CASCADE, verbose_name=_('Review'))
    is_like = models.BooleanField(default=False, verbose_name=_('Is Like'))
    user_obj = models.ForeignKey(User, on_delete=models.CASCADE, verbose_name=_('User'))

    created_at = models.DateTimeField(auto_now_add=True, auto_now=False, verbose_name=_('Created at'))
    update_at = models.DateTimeField(auto_now_add=False, auto_now=True, verbose_name=_('Update at'))

    objects = LikeManager()

    class Meta:
        verbose_name = _('Like')
        verbose_name_plural = _('Likes')

    def __str__(self):
        return "%s" % (self.id)

    def clean(self):

        queryset = Like.objects.review(review_obj=self.review_obj).filter(user_obj=self.user_obj)

        if queryset.count() == 1 and self.id != queryset.first().id:

            raise ValidationError({'user_obj': [_('You can\'t like again.')], 'review_obj': [_('You can\'t like again')]})

    def save(self, *args, **kwargs):
        self.full_clean()
        return super(Like, self).save(*args, **kwargs)

    def get_api_uri(self):
        return reverse('aparnik-api:reviews:like:detail', args=[self.review_obj.id, self.id])


# Base Segment Model
class BaseReviewSegment(BaseSegment):

    objects = BaseSegmentManager()

    def __str__(self):
        return super(BaseReviewSegment, self).__unicode__()

    class Meta:
        verbose_name = _('Review Segment')
        verbose_name_plural = _('Reviews Segments')


def post_save_base_review_receiver(sender, instance, created, *args, **kwargs):

    approve = Setting.objects.get(key='APPROVE_REVIEW').get_value()
    model_obj = instance.model_obj.get_real_instance()
    users = User.objects.none()
    verbose_name = instance._meta.verbose_name
    title = '%s %s جدیدی در %s درج کرد' % (instance.user_obj.get_full_name(), verbose_name, model_obj.get_title())
    description = Truncator(instance.title + ' ' + instance.description).words(30)

    if created:
        if approve:
            instance.is_published = True
            instance.save()
            return
        # admin
        if not instance.user_obj.is_admin:
            users = User.objects.admins()

        users |= model_obj.get_user_for_base_review_notification(instance, created)

    else:
        # notify user change state
        if not approve:
            Notification.objects.send_notification(
                users=[instance.user_obj],
                type=Notification.NOTIFICATION_INFO,
                title='وضعیت %s شما تغییر کرده است.' %verbose_name,
                description=description,
                model_obj=instance,
            )

        # send notification for users
        if instance.parent_obj and instance.is_published:
            # replay before
            users = users | User.objects.filter(
                pk__in=BaseReview.objects.active().filter(parent_obj=instance.parent_obj).values_list('user_obj', flat=True)
            )
            # questions
            users |= User.objects.filter(pk=instance.parent_obj.user_obj.pk)

            users |= model_obj.get_user_for_base_review_notification(instance, created)

    users = users.exclude(pk=instance.user_obj.pk)
    users = users.distinct()

    Notification.objects.send_notification(
        users=users,
        type=Notification.NOTIFICATION_INFO,
        title=title,
        description=description,
        model_obj=instance,
        from_user_obj=instance.user_obj,
    )


# post_save.connect(post_save_base_review_receiver, sender=BaseReview)
# connect all subclasses of base content item too
for subclass in BaseReview.__subclasses__():
    post_save.connect(post_save_base_review_receiver, subclass)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-08-05 16:26


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aparnik_users', '0002_user_passwd'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='verify_mobile',
            field=models.BooleanField(default=False, verbose_name='Verify Mobile'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-12-13 18:47


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pages', '0002_page_english_title'),
        ('segments', '0009_remove_basesegment_pages'),
    ]

    operations = [
        migrations.AddField(
            model_name='basesegment',
            name='pages',
            field=models.ManyToManyField(blank=True, related_name='segment_pages', through='segments.PageSort', to='pages.Page', verbose_name='\\u0635\\u0641\\u062d\\u0647'),
        ),
    ]



from django.conf.urls import url, include

from .views import AddressListAPIView, AddressSetAPIView, AddressCreateAPIView

app_name = 'addresses'

urlpatterns = [
    url(r'^$', AddressListAPIView.as_view(), name='list'),
    url(r'^(?P<model_id>\d+)/set/$', AddressSetAPIView.as_view(), name='set'),
    url(r'^create/$', AddressCreateAPIView.as_view(), name='create'),
]


# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-05-17 16:51


from django.db import migrations

from aparnik.contrib.notifications.models import NotificationType


def add_keys(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    Notification = apps.get_model('notifications', 'Notification')
    for notification in Notification.objects.all():
        notification.notification_send_type = NotificationType.ALL_USER if notification.is_for_all_users else NotificationType.SINGLE_USER
        notification.save()


def remove_keys(apps, schema_editor):
    '''
    We can't import the Post model directly as it may be a newer
    version than this migration expects. We use the historical version.
    '''
    Notification = apps.get_model('notifications', 'Notification')
    for notification in Notification.objects.all():
        notification.is_for_all_users = notification.notification_send_type == NotificationType.ALL_USER
        notification.save()


class Migration(migrations.Migration):

    dependencies = [
        ('notifications', '0002_notification_notification_send_type'),
    ]

    operations = [
        migrations.RunPython(add_keys, reverse_code=remove_keys),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2019-02-01 18:08


from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('products', '0015_auto_20190129_1728'),
        ('orders', '0009_orderitem_description'),
    ]

    operations = [
        migrations.CreateModel(
            name='Subscription',
            fields=[
                ('product_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='products.Product')),
                ('type', models.CharField(choices=[('a', 'All'), ('c', 'Custom')], default='a', max_length=5, verbose_name='Type')),
                ('duration', models.DurationField(default='30 00:00:00', verbose_name='Duration')),
                ('description', models.TextField(blank=True, null=True, verbose_name='Description')),
            ],
            options={
                'verbose_name': 'Subscription',
                'manager_inheritance_from_future': True,
                'verbose_name_plural': 'Subscriptions',
            },
            bases=('products.product',),
        ),
        migrations.CreateModel(
            name='SubscriptionOrder',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('expire_at', models.DateTimeField(verbose_name='Expire at')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('update_at', models.DateTimeField(auto_now=True, verbose_name='Update at')),
                ('order_obj', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='orders.Order', verbose_name='Order')),
                ('subscription_obj', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='subscriptions.Subscription', verbose_name='Subscription')),
            ],
            options={
                'verbose_name': 'Subscription Order',
                'verbose_name_plural': 'Subscriptions Orders',
            },
        ),
        migrations.AddField(
            model_name='subscription',
            name='orders',
            field=models.ManyToManyField(through='subscriptions.SubscriptionOrder', to='orders.Order', verbose_name='Orders'),
        ),
        migrations.AddField(
            model_name='subscription',
            name='products',
            field=models.ManyToManyField(blank=True, related_name='subscriptions', to='products.Product', verbose_name='Products'),
        ),
    ]



from django.conf.urls import url
from .views import BaseModelDetailAPIView, BaseModelListAPIView, TagDetailsAPIView, ShareAPIView

app_name = 'basemodels'

urlpatterns = [
    url(r'^$', BaseModelListAPIView.as_view(), name='list'),
    url(r'^(?P<model_id>\d+)/$', BaseModelDetailAPIView.as_view(), name='details'),
    url(r'^tags/(?P<model_id>\d+)/$', TagDetailsAPIView.as_view(), name='tags-details'),
    url(r'^(?P<model_id>\d+)/share-content/$', ShareAPIView.as_view(), name='share-content'),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-28 17:32


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('products', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='product',
            name='is_free',
            field=models.BooleanField(default=False, verbose_name='Is Free'),
        ),
    ]



from rest_framework import serializers
from aparnik.api.serializers import ModelSerializer

from ..models import TermsAndConditions
from aparnik.contrib.shortblogs.api.serializers import ShortBlogDetailSerializer


class TermsAndCondtionsDestailSerializer(ModelSerializer):

    short_blogs = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(TermsAndCondtionsDestailSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = TermsAndConditions
        fields = [
            'id',
            'short_blogs',
        ]

    def get_short_blogs(self, obj):
        return ShortBlogDetailSerializer(obj.short_blogs, many=True, read_only=True, context=self.context).data



from rest_framework import serializers
from aparnik.api.serializers import ModelSerializer

from aparnik.contrib.province.models import Province, City, Town, Shahrak

class ProvinceListSerializer(ModelSerializer):

    url_details = serializers.SerializerMethodField()
    url_city = serializers.SerializerMethodField()
    url_shahrak = serializers.SerializerMethodField()

    class Meta:
        model = Province
        fields = [
            'id',
            'title',
            'url_details',
            'url_city',
            'url_shahrak',
        ]

    def get_url_details(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_url())

    def get_url_city(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_url_city())

    def get_url_shahrak(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_url_shahrak())

class ProvinceCreateSerializer(ModelSerializer):

    url_details = serializers.SerializerMethodField()
    url_city = serializers.SerializerMethodField()
    url_shahrak = serializers.SerializerMethodField()

    class Meta:
        model = Province

        fields = [
            'id',
            'title',
            'url_details',
            'url_city',
            'url_shahrak',
        ]

        read_only_fields = [
            'url_details',
            'url_city',
            'url_shahrak',
        ]

    def get_url_details(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_url())

    def get_url_city(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_url_city())

    def get_url_shahrak(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_url_shahrak())

class ProvinceDetailSerializer(ModelSerializer):

    url_details = serializers.SerializerMethodField()
    url_city = serializers.SerializerMethodField()
    url_shahrak = serializers.SerializerMethodField()

    class Meta:
        model = Province
        fields = [
            'id',
            'title',
            'url_details',
            'url_city',
            'url_shahrak',
        ]

    def get_url_details(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_url())

    def get_url_city(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_url_city())

    def get_url_shahrak(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_url_shahrak())

# city
class CityListSerializer(ModelSerializer):

    url_details = serializers.SerializerMethodField()
    url_town = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(CityListSerializer, self).__init__(*args, **kwargs)
        self.fields['province'] = ProvinceDetailSerializer(read_only=True, context=kwargs['context'])

    class Meta:
        model = City
        fields = [
            'id',
            'title',
            'url_details',
            'url_town',
        ]

    def get_url_details(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_url())

    def get_url_town(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_url_town())

class CityCreateSerializer(ModelSerializer):

    url_details = serializers.SerializerMethodField()
    url_town = serializers.SerializerMethodField()

    class Meta:
        model = City

        fields = [
            'id',
            'title',
            'province',
            'url_details',
            'url_town',
        ]

        read_only_fields = [
            'url_details',
            'url_town',
        ]

    def get_url_details(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_url())

    def get_url_town(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_url_town())


class CityDetailSerializer(ModelSerializer):

    url_details = serializers.SerializerMethodField()
    url_town = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(CityDetailSerializer, self).__init__(*args, **kwargs)
        self.fields['province'] = ProvinceDetailSerializer(read_only=True, context=kwargs['context'])

    class Meta:
        model = City
        fields = [
            'id',
            'title',
            'url_details',
            'url_town',
        ]

    def get_url_details(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_url())

    def get_url_town(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_url_town())

# shahrak
class ShahrakListSerializer(ModelSerializer):

    url_details = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(ShahrakListSerializer, self).__init__(*args, **kwargs)
        self.fields['province'] = ProvinceDetailSerializer(read_only=True, context=kwargs['context'])

    class Meta:
        model = Shahrak
        fields = [
            'id',
            'title',
            'url_details',
        ]

    def get_url_details(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_url())

class ShahrakCreateSerializer(ModelSerializer):

    url_details = serializers.SerializerMethodField()

    class Meta:
        model = Shahrak

        fields = [
            'id',
            'title',
            'province',
            'url_details',
        ]

        read_only_fields = [
            'url_details',
        ]

    def get_url_details(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_url())


class ShahrakDetailSerializer(ModelSerializer):

    url_details = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(ShahrakDetailSerializer, self).__init__(*args, **kwargs)
        self.fields['province'] = ProvinceDetailSerializer(read_only=True, context=kwargs['context'])

    class Meta:
        model = Shahrak
        fields = [
            'id',
            'title',
            'url_details',
        ]

    def get_url_details(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_url())

# town
class TownListSerializer(ModelSerializer):

    url_details = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(TownListSerializer, self).__init__(*args, **kwargs)
        self.fields['city'] = CityDetailSerializer(read_only=True, context=kwargs['context'])

    class Meta:
        model = Town
        fields = [
            'id',
            'title',
            'url_details',
        ]

    def get_url_details(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_url())

class TownCreateSerializer(ModelSerializer):

    url_details = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(TownCreateSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = Town

        fields = [
            'id',
            'title',
            'city',
            'url_details',
        ]

        read_only_fields = [
            'id',
            'url_details',
        ]

    def get_url_details(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_url())


class TownDetailSerializer(ModelSerializer):

    url_details = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(TownDetailSerializer, self).__init__(*args, **kwargs)
        self.fields['city'] = CityDetailSerializer(read_only=True, context=kwargs['context'])

    class Meta:
        model = Town
        fields = [
            'id',
            'title',
            'url_details',
        ]

    def get_url_details(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_url())


# -*- coding: utf-8 -*-


from rest_framework.generics import RetrieveAPIView, CreateAPIView, UpdateAPIView, ListAPIView
from rest_framework.permissions import (AllowAny, IsAuthenticated)

from .serializers import BaseEducationListPolymorphicSerializer, EducationDetailsSerializers
from ..models import Education, Institute, FieldSubject, Degree


# Create your views here.
class BaseEducationListAPIView(ListAPIView):
    serializer_class = BaseEducationListPolymorphicSerializer
    permission_classes = [AllowAny]

    def get_queryset(self):
        queryset = Education.objects.active()
        return queryset


class InstituteListAPIView(ListAPIView):
    serializer_class = BaseEducationListPolymorphicSerializer
    permission_classes = [AllowAny]

    def get_queryset(self):
        queryset = Institute.objects.active()
        return queryset


class FieldSubjectListAPIView(ListAPIView):
    serializer_class = BaseEducationListPolymorphicSerializer
    permission_classes = [AllowAny]

    def get_queryset(self):
        queryset = FieldSubject.objects.active()
        return queryset


class DegreeListAPIView(ListAPIView):
    serializer_class = BaseEducationListPolymorphicSerializer
    permission_classes = [AllowAny]

    def get_queryset(self):
        queryset = Degree.objects.active()
        return queryset


class EducationCreateAPIView(CreateAPIView):
    serializer_class = EducationDetailsSerializers  # EducationCreateSerializer
    permission_classes = [IsAuthenticated]

    def perform_create(self, serializer, **kwargs):
        user = self.request.user
        dict = {
            'user_obj': user,
        }

        serializer.save(**dict)


class EducationUpdateAPIView(UpdateAPIView):
    serializer_class = EducationDetailsSerializers
    queryset = Education.objects.all()
    permission_classes = [IsAuthenticated]
    lookup_url_kwarg = 'education_id'
    lookup_field = 'basemodel_ptr_id'

    def perform_update(self, serializer, **kwargs):
        user = self.request.user
        dict = {
            'user_obj': user,
        }

        serializer.save(**dict)



# -*- coding: utf-8 -*-


from django.shortcuts import get_object_or_404
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext as _
from rest_framework.generics import RetrieveAPIView, CreateAPIView, UpdateAPIView, ListAPIView
from rest_framework.permissions import (AllowAny, IsAuthenticated)
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework.status import HTTP_200_OK, HTTP_400_BAD_REQUEST
from rest_framework.serializers import as_serializer_error
from rest_framework.exceptions import ValidationError

from ..models import File
from .serializers import FileListPolymorphicSerializer, FileDetailsPolymorphicSerializer
# from courses.models import File, Course
# from libraries.models import Library


class FileListAPIView(ListAPIView):
    serializer_class = FileListPolymorphicSerializer
    permission_classes = [AllowAny]
    # filter_backends = (filters.SearchFilter,)
    search_fields = ('id',)

    def get_queryset(self):
        return File.objects.all()


class FileFileListAPIView(ListAPIView):
    serializer_class = FileListPolymorphicSerializer
    permission_classes = [AllowAny]
    # filter_backends = (filters.SearchFilter,)

    def get_queryset(self):
        # course = get_object_or_404(Course, id=self.request.parser_context['kwargs']['course_id'])
        # return File.objects.filter(course_obj=course)
        # TODO: handle course and etc
        return File.objects.active()

# class BaseFileLibraryListAPIView(ListAPIView):
#     serializer_class = FileListPolymorphicSerializer
#     permission_classes = [AllowAny]
#     # filter_backends = (filters.SearchFilter,)
#
#     def get_queryset(self):
#         queryset = BaseFile.objects.instance_of(Library)
#         return queryset


class FileDownloadRequestAPIView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request, file_id, format=None, *args, **kwargs):
        status = HTTP_400_BAD_REQUEST
        user = self.request.user
        content = {
            'success': False,
            'msg': '',
            'data': {

            },
        }

        try:
            file = get_object_or_404(File.objects.get_this_user(user=user), id=file_id)

            key = self.request.data.get('key', None)
            if not key:
                content['msg'] += _('key is required')

            if content['msg']:
                raise ValidationError({'__all__': [_(content['msg'])]})

            (content["data"]["key"], content['data']['iv'], content["data"]["url_download"]) = file.get_key_url(
                user=user, key=key)
            content["data"]["url_download"] = self.request.build_absolute_uri(content["data"]["url_download"])
            content["success"] = True
            status = HTTP_200_OK
            return Response(content, status=status)
        except Exception as e:
            raise ValidationError(as_serializer_error(e))



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-12-14 13:30


from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('courses', '0010_auto_20181205_1107'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='course',
            name='sort',
        ),
        migrations.RemoveField(
            model_name='coursefile',
            name='sort',
        ),
    ]



# -*- coding: utf-8 -*-


from django import forms
from django.contrib.auth import get_user_model

from .models import Notification, NotificationForSingleUser

User = get_user_model()

# class NotificationForm(forms.ModelForm):
#
#     class Meta:
#         model = Notification
#         fields = ('__all__')
#         widgets = {
#             'user': autocomplete.ModelSelect2(url='users:autocomplete')
#         }

class NotificationForAllUserForm(forms.ModelForm):

    class Meta:
        model = Notification
        fields = ('__all__')
        # widgets = {
        #     'temp': autocomplete.ModelSelect2(url='users:autocomplete')
        # }

# class NotificationForSingleUserForm(forms.ModelForm):
#
#     single_user = forms.ModelChoiceField(
#         queryset=User.objects.all(),
#         widget=autocomplete.ModelSelect2(url='users:autocomplete'),
#         required=True,
#         label="به یک کاربر"
#     )
#
#     class Meta:
#         model = NotificationForSingleUser
#         fields = ('__all__')



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-11-20 10:55


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('questionanswers', '0002_auto_20181026_1745'),
    ]

    operations = [
        migrations.AlterField(
            model_name='qa',
            name='files',
            field=models.ManyToManyField(blank=True, to='filefields.FileField', verbose_name='\\u0641\\u0627\\u06cc\\u0644 \\u0647\\u0627'),
        ),
    ]



# -*- coding: utf-8 -*-


from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _

class InvitationConfig(AppConfig):
        name = 'aparnik.contrib.invitation'
        label = 'invitation'
        verbose_name = _('Invite')



from rest_framework import serializers

from aparnik.utils.utils import is_app_installed
from aparnik.contrib.basemodels.api.serializers import BaseModelListSerializer, BaseModelDetailSerializer
from aparnik.contrib.filefields.api.serializers import FileFieldListSerailizer
from aparnik.contrib.categories.api.serializers import CategoryDetailsPolymorphicSerializer
from ..models import Book, Publisher, WriterTranslator

from aparnik.packages.shops.files.api.serializers import FileListSerializer, FileDetailSerializer


class BookListSerializer(FileListSerializer):

    class Meta:
        model = Book
        fields = FileListSerializer.Meta.fields + [
            'published_date',
        ]


class BookDetailsSerializer(BookListSerializer, FileDetailSerializer):
    categories = serializers.SerializerMethodField()
    publisher = serializers.SerializerMethodField()
    writers = serializers.SerializerMethodField()
    translators = serializers.SerializerMethodField()
    sample_book = serializers.SerializerMethodField()

    class Meta:
        model = Book
        fields = BookListSerializer.Meta.fields + FileDetailSerializer.Meta.fields + [
            'sample_book',
            'publisher',
            'categories',
            'writers',
            'translators',
        ]

    def get_categories(self, obj):
        return CategoryDetailsPolymorphicSerializer(obj.category_obj, many=False, read_only=True, context=self.context).data

    def get_publisher(self, obj):
        return PublisherListSerializer(obj.publisher_obj, many=False, read_only=True, context=self.context).data

    def get_writers(self, obj):
        return WriterTranslatorListSerializer(obj.writers_obj, many=True, read_only=True, context=self.context).data

    def get_translators(self, obj):
        return WriterTranslatorListSerializer(obj.translators_obj, many=True, read_only=True, context=self.context).data

    def get_sample_book(self, obj):
        return FileFieldListSerailizer(obj.sample_book, many=False, read_only=True, context=self.context).data if obj.sample_book else None


class PublisherListSerializer(BaseModelListSerializer):
    url = serializers.SerializerMethodField()

    class Meta:
        model = Publisher
        fields = BaseModelListSerializer.Meta.fields + [
            'url',
            'title',
        ]

    def get_url(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_uri())


class PublisherDetailsSerializer(BaseModelDetailSerializer, PublisherListSerializer):

    class Meta:
        model = Publisher
        fields = BaseModelDetailSerializer.Meta.fields + PublisherListSerializer.Meta.fields + [

        ]


class WriterTranslatorListSerializer(BaseModelListSerializer):
    url = serializers.SerializerMethodField()

    class Meta:
        model = WriterTranslator
        fields = BaseModelListSerializer.Meta.fields + [
            'url',
            'first_name',
            'last_name',
        ]

    def get_url(self, obj):
        return self.context['request'].build_absolute_uri(obj.get_api_uri())


class WriterTranslatorDetailsSerializer(BaseModelDetailSerializer, WriterTranslatorListSerializer):

    class Meta:
        model = WriterTranslator
        fields = BaseModelDetailSerializer.Meta.fields + WriterTranslatorListSerializer.Meta.fields + [

        ]


if is_app_installed('aparnik.contrib.segments'):
    from aparnik.contrib.segments.api.serializers import BaseSegmentListSerializer, BaseSegmentDetailSerializer
    from ..models import BookSegment

    # Base Segment ListSegmentReview
    class SegmentBookListSerializer(BaseSegmentListSerializer):

        class Meta:
            model = BookSegment
            fields = BaseSegmentListSerializer.Meta.fields + [

            ]
            read_only_fields = BaseSegmentListSerializer.Meta.read_only_fields + []


    # Book Segment Details
    class SegmentBookDetailSerializer(BaseSegmentDetailSerializer, SegmentBookListSerializer):

        class Meta:
            model = BookSegment
            fields = SegmentBookListSerializer.Meta.fields + BaseSegmentDetailSerializer.Meta.fields + [

            ]
            read_only_fields = BaseSegmentDetailSerializer.Meta.read_only_fields + SegmentBookListSerializer.Meta.read_only_fields + []


# -*- coding: utf-8 -*-


from django.shortcuts import get_object_or_404
from rest_framework.generics import RetrieveAPIView, CreateAPIView, UpdateAPIView, ListAPIView
from rest_framework.permissions import (AllowAny, IsAuthenticated)
from rest_framework.status import HTTP_200_OK, HTTP_400_BAD_REQUEST
from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework.serializers import as_serializer_error
from rest_framework.exceptions import ValidationError

from aparnik.contrib.basemodels.models import BaseModel
from .serializers import NotifyMeListSerializer
from ..models import NotifyMe


class NotifyMeListAPIView(ListAPIView):
    serializer_class = NotifyMeListSerializer
    permission_classes = [IsAuthenticated]

    def get_queryset(self):
        user = self.request.user
        queryset = NotifyMe.objects.this_user(user)
        # if 'model' in self.request.parser_context['kwargs']:
        #     product_id = self.request.parser_context['kwargs']['product_id']
        #     queryset = ProductSharing.objects.share_this_user(user, product_id)
        return queryset


class NotifyMeSetAPIView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request, model_id, format=None, *args, **kwargs):
        status = HTTP_400_BAD_REQUEST
        user = self.request.user

        content = {
            'is_notify': False,
        }

        model = get_object_or_404(BaseModel.objects.all(), id=model_id)

        try:
            obj = get_object_or_404(NotifyMe.objects.active().filter(model_obj=model), user_obj=user)
            obj.delete()
            status = HTTP_200_OK

        except Exception as e:
            try:
                obj = NotifyMe.objects.create(user_obj=user, model_obj=model, is_active=True)
                content['is_notify'] = True
                status = HTTP_200_OK
            except Exception as e:
                raise ValidationError(as_serializer_error(e))

        return Response(content, status=status)



# -*- coding: utf-8 -*-


from django.contrib import admin
from django.conf import settings

from aparnik.contrib.users.admin import get_update_at
from aparnik.contrib.basemodels.admin import BaseModelAdmin, BaseModelTabularInline
from .models import Page


# Register your models here.
class SegmentSortInline(BaseModelTabularInline):
    model = Page.segment_pages.through
    fields = ('segment_obj', 'sort')
    raw_id_fields = ('segment_obj',)
    extra = 1
    exclude = []

    def __init__(self, *args, **kwargs):
        Klass = SegmentSortInline
        Klass_parent = BaseModelTabularInline

        super(Klass, self).__init__(*args, **kwargs)
        self.exclude = Klass_parent.exclude + self.exclude




class PageAdmin(BaseModelAdmin):
    fields = ['title', 'english_title', 'is_show_in_home']
    list_display = ['title', get_update_at]
    search_fields = ['title', ]
    list_filter = ['created_at']
    raw_id_fields = []
    exclude = []
    dynamic_raw_id_fields = []
    inlines = [SegmentSortInline]

    def __init__(self, *args, **kwargs):
        Klass = PageAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields        
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = Page
    # search_fields = ['email', 'article']
    # list_filter = ['email', 'article', 'created_at']
    # readonly_fields = []
    # actions = []


admin.site.register(Page, PageAdmin)
# admin.site.register(CategoryCourse, CategoryCourseAdmin)
# admin.site.register(CategoryLibrary, CategoryLibraryAdmin)



# -*- coding: utf-8 -*-


from django.db import models
from polymorphic.models import PolymorphicManager, PolymorphicModel
from django.utils.translation import ugettext as _
from django.urls import reverse
from django.core.exceptions import ValidationError
from django.utils.timezone import now

from Crypto.PublicKey import RSA
from Crypto.Cipher import PKCS1_v1_5
from base64 import b64encode, b64decode

from aparnik.contrib.filefields.models import FileField
from aparnik.packages.shops.products.models import Product, ProductManager


# Base FIle MANAGER
class FileManager(ProductManager):

    def get_queryset(self):
        return super(FileManager, self).get_queryset()

    def active(self, user=None):
        return super(FileManager, self).active(user).filter(file_obj__is_lock=False, publish_date__lte=now())

    def get_this_user(self, user):
        # TODO: add handler for user
        return self.get_queryset().all()

    def get_this_type(self, type):
        return self.active().filter(file_obj__type=type)


# Base File class
class File(Product):
    description = models.TextField(null=True, blank=True, verbose_name=_('Description'))
    file_obj = models.ForeignKey(FileField, related_name='shop_file_obj', on_delete=models.CASCADE, verbose_name=_('File'))
    banner = models.ForeignKey(FileField, related_name='shop_file_banner', null=True, blank=True,
                               on_delete=models.CASCADE, verbose_name=_('Banner Image'))
    cover = models.ForeignKey(FileField,  null=True, blank=True, related_name='shop_file_cover', on_delete=models.CASCADE, verbose_name=_('Cover Image'))
    publish_date = models.DateTimeField(default=now, verbose_name=_('Publish Date'))

    objects = FileManager()

    def __str__(self):
        return '%s' % self.id

    class Meta:
        ordering = ['-created_at']
        verbose_name = _('File')
        verbose_name_plural = _('Files')

    def get_api_file_download_request_uri(self):
        return reverse('aparnik-api:shops:files:download-request', args=[self.id])

    def get_key_url(self, user, key):
        if not self.has_permit(user=user):
            raise ValidationError({'__all__': [_('You must buy this File.')]})
        try:
            if self.file_obj.password:
                ciphertext = ''
                if key == 'AAAAaaaa.':
                    ciphertext = self.file_obj.password.encode()
                else:
                    public_key = RSA.importKey(b64decode(key))
                    cipher = PKCS1_v1_5.new(public_key)
                    ciphertext = cipher.encrypt(self.file_obj.password.encode())
                password = b64encode(ciphertext)
            else:
                password = None
            return password, self.file_obj.iv, self.file_obj.url()
        except Exception as e:
            raise ValidationError({'__all__': [_('Wrong key or fail on encryption.')]})

    def get_api_progress_set_uri(self):
        return reverse('aparnik-api:educations:progresses:set', args=[self.id])



from django.contrib.auth import get_user_model
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _
from rest_framework import serializers
from rest_framework.serializers import as_serializer_error
from rest_framework.exceptions import ValidationError

from aparnik.utils.utils import convert_iran_phone_number_to_world_number
from aparnik.contrib.province.api.serializers import CityDetailSerializer
from aparnik.contrib.province.models import City
from aparnik.contrib.invitation.models import Invite
from aparnik.api.serializers import ModelSerializer
from aparnik.utils.formattings import formatprice

User = get_user_model()

user_detail_url = serializers.HyperlinkedIdentityField(view_name='aparnik-api:users:detail', lookup_field='username')
user_update_url = serializers.HyperlinkedIdentityField(view_name='aparnik-api:users:update', lookup_field='username')


class UserSummaryListSerializer(ModelSerializer):
    url = user_detail_url
    avatar = serializers.SerializerMethodField()
    register_date = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(UserSummaryListSerializer, self).__init__(*args, **kwargs)

    class Meta:
        model = User
        fields = [
            'sys_id',
            'url',
            'first_name',
            'last_name',
            'username_mention',
            'sex',
            'avatar',
            'register_date',
        ]

        read_only_fields = [
            'sys_id',
            'url',
            'avatar',
            'register_date',
            'username_mention',
        ]

    def get_avatar(self, obj):
        from aparnik.contrib.filefields.api.serializers import FileFieldListSerailizer
        return FileFieldListSerailizer(obj.avatar, many=False, context=self.context).data if obj.avatar else None

    def get_register_date(self, obj):
        return obj.created_at


class UserDetailSerializer(UserSummaryListSerializer):

    url = user_detail_url
    url_update = user_update_url
    notification_unread_count = serializers.SerializerMethodField()
    aparnik_bon_quantities_accessible = serializers.SerializerMethodField()
    aparnik_bon_price_accessible = serializers.SerializerMethodField()
    aparnik_bon_price_accessible_string = serializers.SerializerMethodField()
    aparnik_bon_price_accessible_and_wallet_string = serializers.SerializerMethodField()
    co_sale = serializers.SerializerMethodField()

    def __init__(self, *args, **kwargs):
        super(UserDetailSerializer, self).__init__(*args, **kwargs)

        if 'context' in kwargs:
            # user = kwargs["context"]["request"].user
            self.fields['current_city'] = CityDetailSerializer(read_only=True, context=kwargs['context'])

    class Meta:
        model = User
        fields = UserSummaryListSerializer.Meta.fields + [
            'url',
            'url_update',
            'uuid',
            'username',
            'birthdate',
            'email',
            'invitation_code',
            'last_login',
            'is_new_registered',
            'wallet',
            'wallet_string',
            'notification_unread_count',
            'aparnik_bon_quantities_accessible',
            'aparnik_bon_price_accessible',
            'aparnik_bon_price_accessible_string',
            'aparnik_bon_price_accessible_and_wallet_string',
            'co_sale',
        ]

        read_only_fields = UserSummaryListSerializer.Meta.read_only_fields + [
            'uuid',
            'last_login',
            'is_new_registered',
            'wallet',
            'wallet_string',
            'notification_unread_count',
            'aparnik_bon_quantities_accessible',
            'aparnik_bon_price_accessible',
            'aparnik_bon_price_accessible_string',
            'aparnik_bon_price_accessible_and_wallet_string',
            'co_sale',
        ]

    def get_notification_unread_count(self, obj):
        return obj.notification_set.unread(user=obj).count()

    def get_aparnik_bon_quantities_accessible(self, obj):
        from aparnik.packages.shops.vouchers.models import Voucher
        return Voucher.objects.quantities_accessible(obj)

    def get_aparnik_bon_price_accessible(self, obj):
        from aparnik.packages.shops.vouchers.models import Voucher
        return Voucher.objects.price_accessible(obj)

    def get_aparnik_bon_price_accessible_string(self, obj):
        from aparnik.packages.shops.vouchers.models import Voucher
        return Voucher.objects.price_accessible_string(obj)

    def get_aparnik_bon_price_accessible_and_wallet_string(self, obj):
        bon_price = self.get_aparnik_bon_price_accessible(obj)
        price = obj.wallet + bon_price
        return '%s' % formatprice.format_price(price)

    def get_co_sale(self, obj):
        from aparnik.packages.shops.cosales.api.serializers import CoSaleUserListSerializer
        return CoSaleUserListSerializer(obj, many=False, read_only=True, context=self.context).data


class UserCreateSerializer(UserDetailSerializer):

    username = serializers.CharField(required=True)

    def __init__(self, *args, **kwargs):
        super(UserCreateSerializer, self).__init__(*args, **kwargs)

        if 'context' in kwargs:
            # user = kwargs["context"]["request"].user
            # self.fields['birth_place'] = CityDetailSerializer(read_only=True, context=kwargs['context'])
            self.fields['current_city'] = CityDetailSerializer(read_only=True, context=kwargs['context'])

    class Meta:
        model = User

        fields = UserDetailSerializer.Meta.fields + [
            'passwd',
        ]

        read_only_fields = UserDetailSerializer.Meta.read_only_fields + [
        ]

        write_only_fields = [
            'passwd',
        ]

    def validate_username(self, value):
        """
        Check that the blog post is about Django.
        """
        username = convert_iran_phone_number_to_world_number(value)
        try:
            User.objects.get(username=username)
            ValidationError({
                'username': [
                    _('This user register before.')
                ]
            })
        except Exception:
            pass
        return username
#


class UserUpdateSerializer(UserDetailSerializer):

    cc = serializers.IntegerField(write_only=True, required=False)
    invited_by = serializers.CharField(write_only=True, required=False)
    avatar_id = serializers.IntegerField(write_only=True, required=False)
    username_mention_update = serializers.CharField(write_only=True, required=False)

    def __init__(self, *args, **kwargs):
        super(UserUpdateSerializer, self).__init__(*args, **kwargs)

        if 'context' in kwargs:
            # user = kwargs["context"]["request"].user
            # self.fields['current_city'] = CityDetailSerializer(read_only=True, context=kwargs['context'])
            pass

    class Meta:
        model = User

        fields = UserDetailSerializer.Meta.fields + [
            'avatar_id',
            'cc',
            'invited_by',
            'passwd',
            'username_mention_update',
        ]

        read_only_fields = UserDetailSerializer.Meta.read_only_fields + [
            'username',
        ]

        write_only_fields = ['passwd', 'username_mention_update']

    def validate_invited_by(self, value):

        self.invite_check(invitation_code=value)
        return value

    def invite_check(self, invitation_code):
        invite = None
        invitation_code = invitation_code
        if invitation_code is not None:
            try:

                invited_by = User.objects.get(invitation_code=invitation_code)
            except:
                raise ValidationError({'invited_by': [_('The invitation code is wrong.'),]})
            invite = Invite(invite=self.instance, invited_by=invited_by)
            invite.clean()

        return invite

    def update(self, instance, validated_data):
        instance.email = validated_data.get('email', instance.email)
        instance.first_name = validated_data.get('first_name', instance.first_name)
        instance.last_name = validated_data.get('last_name', instance.last_name)
        instance.birthdate = validated_data.get('birthdate', instance.birthdate)
        instance.avatar = validated_data.get('avatar', instance.avatar)
        instance.sex = validated_data.get('sex', instance.sex)
        instance.passwd = validated_data.get('passwd', instance.passwd)
        instance.username_mention = validated_data.get('username_mention_update', instance.username_mention)
        if instance.current_city:
            current_city = instance.current_city.id
        else:
            current_city = None
        instance.current_city = City.objects.filter(id=validated_data.get('cc', current_city)).first()

        if instance.avatar:
            avatar = instance.avatar.id
        else:
            avatar = None

        from aparnik.contrib.filefields.models import FileField
        instance.avatar = FileField.objects.filter(id=validated_data.get('avatar_id', avatar)).first()
        try:
            instance.save()
        except Exception as e:
            raise ValidationError(as_serializer_error(e))

        invite = self.invite_check(invitation_code=validated_data.get('invited_by', None))

        if invite:
            invite.save()

        return instance

    def get_current_city(self, obj):
        return CityDetailSerializer(obj.current_city, read_only=True, context=self.context).data

    def get_avatar(self, obj):
        from aparnik.contrib.filefields.api.serializers import FileFieldListSerailizer
        return FileFieldListSerailizer(obj.avatar, many=False, context=self.context).data if obj.avatar else None



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-12-13 16:07


from django.db import migrations


def create_through_relations(apps, schema_editor):
    Segment = apps.get_model('segments', 'BaseSegment')
    SegmentSort = apps.get_model('segments', 'SegmentSort')
    for obj in Segment.objects.all():
        for model_obj in obj.model_obj.all():
            SegmentSort(
                model_obj=model_obj,
                segment_obj=obj,
                sort=0
            ).save()


class Migration(migrations.Migration):

    dependencies = [
        ('segments', '0003_segmentsort'),
    ]

    operations = [
        migrations.RunPython(create_through_relations, reverse_code=migrations.RunPython.noop),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-10-25 15:43


import aparnik.utils.utils
from django.db import migrations, models
import django.db.models.deletion
import s3direct.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('basemodels', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='FileField',
            fields=[
                ('basemodel_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='basemodels.BaseModel')),
                ('file_s3', s3direct.fields.S3DirectField(blank=True, null=True, verbose_name='\\u067e\\u0631\\u0648\\u0646\\u062f\\u0647')),
                ('file_direct', models.FileField(blank=True, null=True, upload_to=aparnik.utils.utils.document_directory_path, verbose_name='\\u067e\\u0631\\u0648\\u0646\\u062f\\u0647')),
                ('size', models.IntegerField(verbose_name='File Size')),
                ('type', models.CharField(choices=[('M', 'Movie'), ('V', 'Voice'), ('P', 'PDF'), ('I', '\\u062a\\u0635\\u0648\\u06cc\\u0631')], max_length=1, verbose_name='File Type')),
            ],
            options={
                'verbose_name': 'File Field',
                'verbose_name_plural': 'Files Field',
            },
            bases=('basemodels.basemodel',),
        ),
    ]



# -*- coding: utf-8 -*-


from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from jalali_date.admin import ModelAdminJalaliMixin
from jalali_date import datetime2jalali

from aparnik.settings import aparnik_settings
from aparnik.contrib.users import admin as aparnik_admin
from aparnik.contrib.basemodels.admin import BaseModelAdmin, BaseModelStackedInline
from aparnik.contrib.users.admin import get_user_search_fields
from .models import Teacher, Dossier

# Register your models here.


def get_start_date(obj):
    if obj.start_date:
        return datetime2jalali(obj.start_date).strftime(aparnik_settings.JALALI_FORMAT)
    return obj.start_date


def get_finish_date(obj):
    if obj.finish_date:
        return datetime2jalali(obj.finish_date).strftime(aparnik_settings.JALALI_FORMAT)
    return obj.finish_date


def get_first_name(obj):
    return obj.user_obj.first_name


def get_last_name(obj):
    return obj.user_obj.last_name


get_start_date.short_description = _('Start Date')
get_start_date.admin_order_field = 'start_date'

get_last_name.short_description = _('Last Name')
get_last_name.admin_order_field = 'user_obj__last_name'

get_first_name.short_description = _('First Name')
get_first_name.admin_order_field = 'user_obj__first_name'


class DossierInline(ModelAdminJalaliMixin, BaseModelStackedInline):
    model = Dossier
    extra = 1
    fk_name = 'teacher_obj'
    exclude = []

    def __init__(self, *args, **kwargs):
        Klass = DossierInline
        Klass_parent = BaseModelStackedInline

        super(Klass, self).__init__(*args, **kwargs)
        self.exclude = Klass_parent.exclude + self.exclude



class TeacherAdmin(BaseModelAdmin):
    fields = ['user_obj', 'slider_segment_obj', 'biography', 'start_date']
    list_display = ['user_obj', get_first_name, get_last_name, get_start_date, aparnik_admin.get_update_at]
    list_filter = []
    search_fields = get_user_search_fields('user_obj') + []
    exclude = []
    dynamic_raw_id_fields = ['slider_segment_obj']
    inlines = [
        DossierInline,
    ]
    raw_id_fields = ['user_obj', ]

    def __init__(self, *args, **kwargs):
        Klass = TeacherAdmin
        Klass_parent = BaseModelAdmin

        super(Klass, self).__init__(*args, **kwargs)
        self.fields = Klass_parent.fields + self.fields
        self.list_display = Klass_parent.list_display + self.list_display
        self.list_filter = Klass_parent.list_filter + self.list_filter
        self.search_fields = Klass_parent.search_fields + self.search_fields
        self.exclude = Klass_parent.exclude + self.exclude
        self.dynamic_raw_id_fields = Klass_parent.dynamic_raw_id_fields + self.dynamic_raw_id_fields        
        self.raw_id_fields = Klass_parent.raw_id_fields + self.raw_id_fields

    class Meta:
        model = Teacher


admin.site.register(Teacher, TeacherAdmin)
